@@ -20,6 +20,7 @@
 #include <GUIBase.h>
 #include <assert.h>
 #include <GraphicContext.h>
+#include <cmath>
 
 #define SCREEN_ANIMATION_FRAME_COUNT 10
 
@@ -27,6 +28,16 @@ using namespace GAGCore;
 
 namespace GAGGUI
 {
+	namespace ColorTheme
+	{
+		// for green theme
+		Color frontColor = Color(255, 255, 255);
+		Color frontFrameColor = Color(0, 200, 100);
+		Color listSelectedElementColor = Color(170, 170, 240);
+		Color backColor = Color(0, 0, 0);
+		Color backOverlayColor = Color(0, 0, 40);
+	}
+	
 	// this function support base unicode (UCS16)
 	void UCS16toUTF8(Uint16 ucs16, char utf8[4])
 	{
@@ -191,6 +202,11 @@ namespace GAGGUI
 				*sx=x+((screenw-640)>>1);
 				*sw=w;
 				break;
+				
+			case ALIGN_CENTERED:
+				*sx = (screenw - w) >> 1;
+				*sw = w;
+				break;
 	
 			default:
 				assert(false);
@@ -217,6 +233,11 @@ namespace GAGGUI
 				*sy=y+((screenh-480)>>1);
 				*sh=h;
 				break;
+				
+			case ALIGN_CENTERED:
+				*sy = (screenh - h) >> 1;
+				*sh = h;
+				break;
 	
 			default:
 				assert(false);
@@ -277,11 +298,8 @@ namespace GAGGUI
 		}
 	}
 	
-	void HighlightableWidget::paint(void)
+	unsigned HighlightableWidget::getNextHighlightValue(void)
 	{
-		int x, y, w, h;
-		getScreenPos(&x, &y, &w, &h);
-		
 		float actHighlight;
 		if (actAnimationTime > 0)
 		{
@@ -291,12 +309,21 @@ namespace GAGGUI
 		}
 		else
 			actHighlight = nextHighlightValue;
+			
+		return static_cast<unsigned>(255.0 * actHighlight);
+	}
+	
+	void HighlightableWidget::paint(void)
+	{
+		int x, y, w, h;
+		getScreenPos(&x, &y, &w, &h);
 		
-		parent->getSurface()->drawRect(x, y, w, h, 200, 200, 200);
-		if (actHighlight > 0)
+		parent->getSurface()->drawRect(x, y, w, h, ColorTheme::frontFrameColor);
+		
+		unsigned val = getNextHighlightValue();
+		if (val > 0)
 		{
-			unsigned val = static_cast<unsigned>(255.0 * actHighlight);
-			parent->getSurface()->drawRect(x+1, y+1, w-2, h-2, 255, 255, 255, val);
+			parent->getSurface()->drawRect(x+1, y+1, w-2, h-2, ColorTheme::frontColor.applyAlpha(val));
 		}
 	}
 	
@@ -469,7 +496,7 @@ namespace GAGGUI
 	
 	void Screen::paint(void)
 	{
-		gfx->drawFilledRect(0, 0, getW(), getH(), 0, 0, 0);
+		gfx->drawFilledRect(0, 0, getW(), getH(), ColorTheme::backColor);
 	}
 	
 	int Screen::getW(void)
@@ -531,6 +558,6 @@ namespace GAGGUI
 	
 	void OverlayScreen::paint(void)
 	{
-		gfx->drawFilledRect(0, 0, getW(), getH(), 0, 0, 40);
+		gfx->drawFilledRect(0, 0, getW(), getH(), ColorTheme::backOverlayColor);
 	}
 }
