@@ -1005,6 +1005,10 @@ Order *AICastor::controlUpgrades()
 		return NULL;
 	if (b->maxUnitWorking<1)
 		return new OrderModifyBuilding(b->gid, 1);
+	int numberOfFreeWorkers = team->stats.getLatestStat()->isFree[WORKER];
+	int numberOfAbleWorkers = team->stats.getLatestStat()->upgradeState[BUILD][b->type->level];
+	if (numberOfAbleWorkers <= 2 || numberOfFreeWorkers <= 4 || numberOfAbleWorkers <= (numberOfFreeWorkers/8))
+		return NULL;
 	// Is it any repair:
 	if (!b->type->isBuildingSite)
 	{
@@ -2546,7 +2550,7 @@ void AICastor::computeWheatCareMap()
 			else
 				wheatCareMap[0][i]=8;
 		}
-	map->updateGlobalGradient(wheatCareMap[0]);
+	map->updateGlobalGradientSlow(wheatCareMap[0]);
 }
 
 void AICastor::computeWheatGrowthMap()
@@ -2569,7 +2573,7 @@ void AICastor::computeWheatGrowthMap()
 		if (wheatGradient[i]==255)
 			wheatGrowthMap[i]=1+(hydratationMap[i]>>3);
 	
-	map->updateGlobalGradient(wheatGrowthMap);
+	map->updateGlobalGradientSlow(wheatGrowthMap);
 	
 	for (size_t i=0; i<size; i++)
 	{
@@ -2706,7 +2710,7 @@ void AICastor::computeEnemyRangeMap()
 		}
 	}
 	
-	map->updateGlobalGradient(gradient);
+	map->updateGlobalGradientSlow(gradient);
 }
 
 void AICastor::computeEnemyWarriorsMap()
@@ -2739,7 +2743,7 @@ void AICastor::computeEnemyWarriorsMap()
 			continue;
 		gradient[i]=32;
 	}
-	map->updateGlobalGradient(gradient);
+	map->updateGlobalGradientSlow(gradient);
 }
 
 Order *AICastor::findGoodBuilding(Sint32 typeNum, bool food, bool defense, bool critical)
