@@ -527,32 +527,24 @@ void Building::neededRessources(int needs[MAX_NB_RESSOURCES])
 void Building::wishedRessources(int needs[MAX_NB_RESSOURCES])
 {
 	 // we balance the system with Units working on it:
-	for (int ri=0; ri<MAX_NB_RESSOURCES; ri++)
-		needs[ri]=(type->maxRessource[ri]-ressources[ri]+1-type->multiplierRessource[ri]);
-	for (std::list<Unit *>::iterator ui=unitsWorking.begin(); ui!=unitsWorking.end(); ++ui)
-		if ((*ui)->destinationPurprose>=0)
+	for (int ri = 0; ri < MAX_NB_RESSOURCES; ri++)
+		needs[ri] = (2 * (type->maxRessource[ri] - ressources[ri])) / (type->multiplierRessource[ri]);
+	for (std::list<Unit *>::iterator ui = unitsWorking.begin(); ui != unitsWorking.end(); ++ui)
+		if ((*ui)->destinationPurprose >= 0)
 		{
-			assert((*ui)->destinationPurprose<MAX_NB_RESSOURCES);
+			assert((*ui)->destinationPurprose < MAX_NB_RESSOURCES);
 			needs[(*ui)->destinationPurprose]--;
 		}
-	for (int i=0; i<20; i++)
-	{
-		for (int ri=0; ri<MAX_NB_RESSOURCES; ri++)
-			if (needs[ri]>0)
-				return;
-		for (int ri=0; ri<MAX_NB_RESSOURCES; ri++)
-			needs[ri]+=(type->maxRessource[ri]-ressources[ri]+1-type->multiplierRessource[ri]);
-	}
 }
 
 int Building::neededRessource(int r)
 {
-	assert(r>=0);
-	int need=type->maxRessource[r]-ressources[r]+1-type->multiplierRessource[r];
-	if (need>0)
+	assert(r >= 0);
+	int need = type->maxRessource[r] - ressources[r] + 1 - type->multiplierRessource[r];
+	if (need > 0)
 		return need;
 	else
-		return false;
+		return 0;
 }
 
 void Building::launchConstruction(void)
@@ -715,6 +707,7 @@ void Building::setMapDiscovered(void)
 		owner->map->setMapDiscovered(posX-vr, posY-vr, type->width+vr*2, type->height+vr*2, owner->sharedVisionFood);
 	else
 		owner->map->setMapDiscovered(posX-vr, posY-vr, type->width+vr*2, type->height+vr*2, owner->sharedVisionOther);
+	owner->map->setMapExploredByBuilding(posX-vr, posY-vr, type->width+vr*2, type->height+vr*2, owner->teamNumber);
 }
 
 void Building::launchDelete(void)
@@ -1076,7 +1069,7 @@ void Building::updateBuildingSite(void)
 			owner->clearingFlags.push_back(this);
 		
 		setMapDiscovered();
-		owner->setEvent(getMidX(), getMidY(), Team::BUILDING_FINISHED_EVENT, gid);
+		owner->setEvent(getMidX(), getMidY(), Team::BUILDING_FINISHED_EVENT, gid, owner->teamNumber);
 
 		// we need to do an update again
 		updateCallLists();
@@ -1279,7 +1272,19 @@ void Building::removeSubscribers(void)
 
 bool Building::fullWorking(void)
 {
-	return ((signed)unitsWorking.size()>=maxUnitWorking);
+	return ((Sint32)unitsWorking.size() >= maxUnitWorking);
+}
+
+bool Building::enoughWorking(void)
+{
+	int neededRessourcesSum = 0;
+	for (size_t ri = 0; ri < MAX_RESSOURCES; ri++)
+	{
+		int neededRessources = (type->maxRessource[ri] - ressources[ri]) / type->multiplierRessource[ri];
+		if (neededRessources > 0)
+			neededRessourcesSum += neededRessources;
+	}
+	return ((int)unitsWorking.size() >= 2 * neededRessourcesSum);
 }
 
 bool Building::fullInside(void)
@@ -1294,13 +1299,13 @@ void Building::subscribeToBringRessourcesStep()
 {
 	if (subscriptionWorkingTimer>0)
 		subscriptionWorkingTimer++;
-	if (fullWorking())
+	if (fullWorking() || enoughWorking())
 	{
 		for (std::list<Unit *>::iterator it=unitsWorkingSubscribe.begin(); it!=unitsWorkingSubscribe.end(); ++it)
 			(*it)->standardRandomActivity();
 		unitsWorkingSubscribe.clear();
 		if (verbose)
-			printf("...fullWorking()\n");
+			printf("...enoughWorking()\n");
 		return;
 	}
 	
@@ -1556,7 +1561,7 @@ void Building::subscribeForFlagingStep()
 			/* To choose a good unit, we get a composition of things:
 			1-the closest the unit is, the better it is.
 			2-the less the unit is hungry, the better it is.
-			2-the more hp the unit has, the better it is.
+			3-the more hp the unit has, the better it is.
 			*/
 			if (type->zonable[EXPLORER])
 			{
@@ -1570,7 +1575,7 @@ void Building::subscribeForFlagingStep()
 					int dist=map->warpDistSquare(unit->posX, unit->posY, posX, posY);
 					if (dist<timeLeft)
 					{
-						int value=dist-timeLeft-hp;
+						int value=dist-2*timeLeft-2*hp;
 						if (value<minValue)
 						{
 							minValue=value;
@@ -1579,7 +1584,26 @@ void Building::subscribeForFlagingStep()
 					}
 				}
 			}
-			else
+			else if (type->zonable[WARRIOR])
+			{
+				for (std::list<Unit *>::iterator it=unitsWorkingSubscribe.begin(); it!=unitsWorkingSubscribe.end(); it++)
+				{
+					Unit *unit=(*it);
+					int timeLeft=unit->hungry/unit->race->hungryness;
+					int hp=(unit->hp<<4)/unit->race->unitTypes[0][0].performance[HP];
+					int dist;
+					if (map->buildingAvailable(this, unit->performance[SWIM]>0, unit->posX, unit->posY, &dist) && (dist<timeLeft))
+					{
+						int value=dist-2*timeLeft-2*hp;
+						if (value<minValue)
+						{
+							minValue=value;
+							choosen=unit;
+						}
+					}
+				}
+			}
+			else if (type->zonable[WORKER])
 			{
 				for (std::list<Unit *>::iterator it=unitsWorkingSubscribe.begin(); it!=unitsWorkingSubscribe.end(); it++)
 				{
@@ -1601,6 +1625,8 @@ void Building::subscribeForFlagingStep()
 					}
 				}
 			}
+			else
+				assert(false);
 			
 			if (choosen)
 			{
@@ -1797,19 +1823,23 @@ void Building::turretStep(void)
 	enum TargetType
 	{
 		TARGETTYPE_NONE,
-		TARGETTYPE_WARRIOR,
-		TARGETTYPE_WORKER,
 		TARGETTYPE_BUILDING,
+		TARGETTYPE_WORKER,
+		TARGETTYPE_WARRIOR,
 	};
 	// The type of the best target we have found up to now
 	TargetType targetFound = TARGETTYPE_NONE;
 	// The score of the best target we have found up to now
-	int bestScore = 0;
+	int bestScore = INT_MIN;
+	// The number of ticks before the unit may move away
+	int bestTicks = 0;
 	// The position of the best target we have found up to now
 	int bestTargetX = 0, bestTargetY=0;
 	
 	for (int i=0; i<=range ; i++)
 	{
+		// The number of ticks before the bullet hits the target at range "i".
+		int ticksToHit = ((i << 5) + ((type->width) << 4)) / (type->shootSpeed>>8);
 		for (int j=0; j<=i ; j++)
 		{
 			for (int k=0; k<8; k++)
@@ -1866,21 +1896,33 @@ void Building::turretStep(void)
 						Unit *testUnit = owner->game->teams[otherTeam]->myUnits[targetID];
 						if ((owner->sharedVisionExchange & otherTeamMask) == 0)
 						{
-							int targetScore;
+							int targetTicks = (256 - testUnit->delta) / testUnit->speed;
+							// skip this unit if it will move away too soon.
+							if (targetTicks <= ticksToHit)
+								continue;
 							// shoot warrior first, then workers if no warrior
 							if (testUnit->typeNum == WARRIOR)
 							{
-								// score is proportional to speed, probability to hit, attack strength and actual damage being conducted
-								if (testUnit->movement == Unit::MOV_ATTACKING_TARGET)
-									targetScore = 1000 + testUnit->getRealAttackStrength();
+								int targetOffense = (testUnit->getRealAttackStrength() * testUnit->performance[ATTACK_SPEED]); // 88 to 1024
+								int targetWeakeness = 0; // 0 to 512
+								if (testUnit->hp > 0)
+								{
+									if (testUnit->hp < type->shootDamage) // hahaha, how mean!
+										targetWeakeness = 512;
+									else
+										targetWeakeness = 256 / testUnit->hp;
+								}
+								int targetProximity = 0; // 0 to 512
+								if (i <= 0)
+									targetProximity = 512;
 								else
-									targetScore = testUnit->speed/(256-testUnit->delta) + 10*testUnit->getRealAttackStrength();
-								// adjust score for range
-								targetScore = ((range+1)*targetScore)/(i+1);
-								// anything else or warriors with lower scores are overriden
-								if ((targetFound != TARGETTYPE_WARRIOR) || (targetScore > bestScore))
+									targetProximity = (256 / i);
+								int targetScore = targetOffense + targetWeakeness + targetProximity;
+								// lower scores are overriden
+								if (targetScore > bestScore)
 								{
 									bestScore = targetScore;
+									bestTicks = targetTicks;
 									bestTargetX = targetX;
 									bestTargetY = targetY;
 									targetFound = TARGETTYPE_WARRIOR;
@@ -1888,14 +1930,13 @@ void Building::turretStep(void)
 							}
 							else if ((targetFound != TARGETTYPE_WARRIOR) && (testUnit->typeNum == WORKER))
 							{
-								// score is proportional to speed and probability to hit
-								targetScore = testUnit->speed/(256-testUnit->delta);
 								// adjust score for range
-								targetScore = ((range+1)*targetScore)/(i+1);
-								// building or workers with lower scores are overriden
-								if ((targetFound != TARGETTYPE_WORKER) || (targetScore > bestScore))
+								int targetScore = - testUnit->hp;
+								// lower scores are overriden
+								if (targetScore > bestScore)
 								{
 									bestScore = targetScore;
+									bestTicks = targetTicks;
 									bestTargetX = targetX;
 									bestTargetY = targetY;
 									targetFound = TARGETTYPE_WORKER;
@@ -1916,10 +1957,11 @@ void Building::turretStep(void)
 						if (enemies & otherTeamMask)
 						{
 							// adjust score for range
-							int targetScore = (range+1)/(i+1); 
+							int targetScore = - i;
 							if (targetScore > bestScore)
 							{
 								bestScore = targetScore;
+								bestTicks = 256;
 								bestTargetX = targetX;
 								bestTargetY = targetY;
 								targetFound = TARGETTYPE_BUILDING;
@@ -1990,12 +2032,14 @@ void Building::turretStep(void)
 				return;
 			}
 		}
-
-		Bullet *b = new Bullet(px, py, speedX, speedY, ticksLeft, type->shootDamage, bestTargetX, bestTargetY, posX-1, posY-1, type->width+2, type->height+2);
-		s->bullets.push_front(b);
-
-		bullets--;
-		shootingCooldown = SHOOTING_COOLDOWN_MAX;
+		
+		if (ticksLeft < bestTicks)
+		{
+			Bullet *b = new Bullet(px, py, speedX, speedY, ticksLeft, type->shootDamage, bestTargetX, bestTargetY, posX-1, posY-1, type->width+2, type->height+2);
+			s->bullets.push_front(b);
+			bullets--;
+			shootingCooldown = SHOOTING_COOLDOWN_MAX;
+		}
 	}
 
 }
@@ -2300,14 +2344,16 @@ Uint32 Building::eatOnce(Uint32 *mask)
 	return fruitCount;
 }
 
-int Building::availableHappynessLevel()
+int Building::availableHappynessLevel(bool guarantee)
 {
-	int inside=(int)unitsInside.size();
-	if (ressources[CORN]<=inside)
+	int inside = (int)unitsInside.size();
+	if (guarantee)
+		inside += unitsInsideSubscribe.size();
+	if (ressources[CORN] <= inside)
 		return 0;
-	int happyness=1;
-	for (int i=0; i<HAPPYNESS_COUNT; i++)
-		if (ressources[i+HAPPYNESS_BASE]>inside)
+	int happyness = 1;
+	for (int i = 0; i < HAPPYNESS_COUNT; i++)
+		if (ressources[i + HAPPYNESS_BASE]  >inside)
 			happyness++;
 	return happyness;
 }
@@ -2370,64 +2416,64 @@ void Building::integrity()
 	}
 }
 
-Uint32 Building::checkSum(std::list<Uint32> *checkSumsList)
+Uint32 Building::checkSum(std::vector<Uint32> *checkSumsVector)
 {
 	int cs=0;
 	
 	cs^=typeNum;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [0]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [0]
 
 	cs^=buildingState;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [1]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [1]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=maxUnitWorking;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [2]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [2]
 	cs^=maxUnitWorkingPreferred;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [3]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [3]
 	cs^=unitsWorking.size();
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [4]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [4]
 	cs^=maxUnitInside;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [5]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [5]
 	cs^=unitsInside.size();
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [6]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [6]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=posX;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [7]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [7]
 	cs^=posY;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [8]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [8]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=unitStayRange;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [9]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [9]
 
 	for (int i=0; i<MAX_RESSOURCES; i++)
 		cs^=ressources[i];
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [10]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [10]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=hp;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [11]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [11]
 
 	cs^=productionTimeout;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [12]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [12]
 	cs^=totalRatio;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [13]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [13]
 	
 	for (int i=0; i<NB_UNIT_TYPE; i++)
 	{
@@ -2435,27 +2481,27 @@ Uint32 Building::checkSum(std::list<Uint32> *checkSumsList)
 		cs^=percentUsed[i];
 		cs=(cs<<31)|(cs>>1);
 	}
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [14]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [14]
 
 	cs^=shootingStep;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [15]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [15]
 	cs^=shootingCooldown;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [16]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [16]
 	cs^=bullets;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [17]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [17]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=seenByMask;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [18]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [18]
 	
 	cs^=gid;
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [19]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [19]
 	
 	return cs;
 }
