@@ -1020,7 +1020,7 @@ void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const char* name
 	}
 	else
 	{
-		SAVE_OFFSET(stream, 16);
+		SAVE_OFFSET(stream, 16, "gameOffset");
 		stream->write("GaBe", 4, "signatureStart");
 
 		stream->writeUint32(stepCounter, "stepCounter");
@@ -1029,7 +1029,7 @@ void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const char* name
 		stream->writeUint32(getSyncRandSeedC(), "SyncRandSeedC");
 		stream->write("GaSy", 4, "signatureAfterSyncRand");
 
-		SAVE_OFFSET(stream, 20);
+		SAVE_OFFSET(stream, 20, "teamsOffset");
 		stream->writeEnterSection("teams");
 		for (int i=0; i<session.numberOfTeam; ++i)
 		{
@@ -1040,11 +1040,11 @@ void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const char* name
 		stream->writeLeaveSection();
 		stream->write("GaTe", 4, "signatureAfterTeams");
 
-		SAVE_OFFSET(stream, 28);
+		SAVE_OFFSET(stream, 28, "mapOffset");
 		map.save(stream);
 		stream->write("GaMa", 4, "signatureAfterMap");
 		
-		SAVE_OFFSET(stream, 24);
+		SAVE_OFFSET(stream, 24, "playersOffset");
 		stream->writeEnterSection("players");
 		for (int i=0; i<session.numberOfPlayer; ++i)
 		{
@@ -1055,7 +1055,7 @@ void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const char* name
 		stream->writeLeaveSection();
 		stream->write("GaPl", 4, "signatureAfterPlayers");
 
-		SAVE_OFFSET(stream, 32);
+		SAVE_OFFSET(stream, 32, "mapScriptOffset");
 		script.save(stream);
 		
 		stream->writeText(nextMap, "nextMap");
@@ -1583,7 +1583,7 @@ void Game::drawPointBar(int x, int y, BarOrientation orientation, int maxLength,
 		assert(false);
 }
 
-void Game::drawUnit(int x, int y, Uint16 gid, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+void Game::drawUnit(int x, int y, Uint16 gid, int viewportX, int viewportY, int screenW, int screenH, int localTeam, Uint32 drawOptions)
 {
 	int id=Unit::GIDtoID(gid);
 	int team=Unit::GIDtoTeam(gid);
@@ -1706,17 +1706,21 @@ void Game::drawUnit(int x, int y, Uint16 gid, int viewportX, int viewportY, int
 			globalContainer->gfx->drawSprite(px+24, py, globalContainer->ressourceMini, unit->caryedRessource);
 	}
 	if (((drawOptions & DRAW_PATH_LINE) != 0) && (unit->owner->sharedVisionOther & teams[localTeam]->me))
-		if (unit->displacement==Unit::DIS_GOING_TO_FLAG
+		if (unit->validTarget)
+		/*TODO: remove this comment when we see the new code works
+			if (unit->displacement==Unit::DIS_GOING_TO_FLAG
 			|| unit->displacement==Unit::DIS_GOING_TO_RESSOURCE
 			|| unit->displacement==Unit::DIS_GOING_TO_BUILDING
-			|| (unit->displacement==Unit::DIS_ATTACKING_AROUND
-				&& (unit->movement==Unit::MOV_GOING_DXDY)))
+			|| (unit->displacement==Unit::DIS_ATTACKING_AROUND && unit->movement==Unit::MOV_GOING_DXDY))*/
 		{
 			int lsx, lsy, ldx, ldy;
 			lsx=px+16;
 			lsy=py+16;
-			map.mapCaseToDisplayable(unit->targetX, unit->targetY, &ldx, &ldy, viewportX, viewportY);
-			globalContainer->gfx->drawLine(lsx, lsy, ldx+16, ldy+16, 250, 250, 250);
+			map.mapCaseToDisplayableVector(unit->targetX, unit->targetY, &ldx, &ldy, viewportX, viewportY, screenW, screenH);
+			if (globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX)
+				globalContainer->gfx->drawLine(lsx, lsy, ldx+16, ldy+16, 250, 250, 250);
+			else
+				globalContainer->gfx->drawLine(lsx, lsy, ldx+16, ldy+16, 250, 250, 250, 128);
 		}
 	if (drawOptions & DRAW_ACCESSIBILITY)
 	{
@@ -1853,25 +1857,34 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 		{
 			Uint16 gid=map.getGroundUnit(x+viewportX, y+viewportY);
 			if (gid!=NOGUID)
-				drawUnit(x, y, gid, viewportX, viewportY, localTeam, drawOptions);
+				drawUnit(x, y, gid, viewportX, viewportY, (sw>>5), (sh>>5), localTeam, drawOptions);
 		}
 	
 	// We draw debug area:
 	if (false)
 		for (int y=top-1; y<=bot; y++)
 			for (int x=left-1; x<=right; x++)
+			{
+				//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((AICastor *)players[1]->ai->aiImplementation)->wheatCareMap[0][(x+viewportX)+(y+viewportY)*map.w]);
+				
+				//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((AICastor *)players[1]->ai->aiImplementation)->notGrassMap[(x+viewportX)+(y+viewportY)*map.w]);
+				
+				globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.getExplored(x+viewportX, y+viewportY, 0));
+				
+				//((AICastor *)players[0].ai->aiImplementation)->wheatCareMap
+			}
 				//if (map.getForbidden(x+viewportX, y+viewportY))
-				{
+				//{
 					//if (!map.isFreeForGroundUnit(x+viewportX, y+viewportY, 1, 1))
 					//	globalContainer->gfx->drawRect(x<<5, y<<5, 32, 32, 255, 16, 32);
 					//globalContainer->gfx->drawRect(2+(x<<5), 2+(y<<5), 28, 28, 255, 16, 32);
 					//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.getGradient(1, 5, 0, x+viewportX, y+viewportY));
 					//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.getGradient(0, STONE, 1, x+viewportX, y+viewportY));
-					globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.forbiddenGradient[0][0][(x+viewportX)+(y+viewportY)*map.w]);
+					//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.forbiddenGradient[0][0][(x+viewportX)+(y+viewportY)*map.w]);
 					
 					//globalContainer->gfx->drawString((x<<5), (y<<5)+16, globalContainer->littleFont, ((x+viewportX)&(map.getMaskW())));
 					//globalContainer->gfx->drawString((x<<5)+16, (y<<5)+8, globalContainer->littleFont, ((y+viewportY)&(map.getMaskH())));
-				}
+				//}
 
 	// We draw debug area:
 	if (false)
@@ -2149,7 +2162,7 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 		{
 			Uint16 gid=map.getAirUnit(x+viewportX, y+viewportY);
 			if (gid!=NOGUID)
-				drawUnit(x, y, gid, viewportX, viewportY, localTeam, drawOptions);
+				drawUnit(x, y, gid, viewportX, viewportY, (sw>>5), (sh>>5), localTeam, drawOptions);
 		}
 
 	// Let's paint the bullets and explosions
@@ -2195,14 +2208,31 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 		// explosions
 		for (std::list<BulletExplosion *>::iterator it=s->explosions.begin();it!=s->explosions.end();it++)
 		{
-			int x, y;
-			map.mapCaseToDisplayable((*it)->x, (*it)->y, &x, &y, viewportX, viewportY);
-			Sprite *img = globalContainer->bulletExplosion;
-			int frame = globalContainer->bulletExplosion->getFrameCount() - (*it)->ticksLeft - 1;
-			int decX = globalContainer->bulletExplosion->getW(frame)>>1;
-			int decY = globalContainer->bulletExplosion->getH(frame)>>1;
-			globalContainer->gfx->drawSprite(x+16-decX, y+16-decY, img, frame);
-			//std::cout << "Explosion at (" << x << "," << y << ") frame " << frame <<std::endl;
+			if (map.isFOWDiscovered((*it)->x, (*it)->y, teams[localTeam]->me))
+			{
+				int x, y;
+				map.mapCaseToDisplayable((*it)->x, (*it)->y, &x, &y, viewportX, viewportY);
+				int frame = globalContainer->bulletExplosion->getFrameCount() - (*it)->ticksLeft - 1;
+				int decX = globalContainer->bulletExplosion->getW(frame)>>1;
+				int decY = globalContainer->bulletExplosion->getH(frame)>>1;
+				globalContainer->gfx->drawSprite(x+16-decX, y+16-decY, globalContainer->bulletExplosion, frame);
+			}
+		}
+		// death animations
+		for (std::list<UnitDeathAnimation *>::iterator it=s->deathAnimations.begin();it!=s->deathAnimations.end();++it)
+		{
+			if (map.isFOWDiscovered((*it)->x, (*it)->y, teams[localTeam]->me))
+			{
+				int x, y;
+				map.mapCaseToDisplayable((*it)->x, (*it)->y, &x, &y, viewportX, viewportY);
+				int frame = globalContainer->deathAnimation->getFrameCount() - (*it)->ticksLeft - 1;
+				int decX = globalContainer->deathAnimation->getW(frame)>>1;
+				int decY = globalContainer->deathAnimation->getH(frame)>>1;
+				Team *team = (*it)->team;
+				
+				globalContainer->deathAnimation->setBaseColor(team->colorR, team->colorG, team->colorB);
+				globalContainer->gfx->drawSprite(x+16-decX, y+16-decY-frame, globalContainer->deathAnimation, frame);
+			}
 		}
 	}
 	
@@ -2664,40 +2694,40 @@ void Game::renderMiniMap(int localTeam, const bool useMapDiscovered, int step, i
 		}*/
 }
 
-Uint32 Game::checkSum(std::list<Uint32> *checkSumsList, std::list<Uint32> *checkSumsListForBuildings, std::list<Uint32> *checkSumsListForUnits)
+Uint32 Game::checkSum(std::vector<Uint32> *checkSumsVector, std::vector<Uint32> *checkSumsVectorForBuildings, std::vector<Uint32> *checkSumsVectorForUnits)
 {
 	Uint32 cs=0;
 
 	Uint32 sessionCs=session.checkSum();
 	cs^=sessionCs;
-	if (checkSumsList)
-		checkSumsList->push_back(sessionCs);// [0]
+	if (checkSumsVector)
+		checkSumsVector->push_back(sessionCs);// [0]
 
 	cs=(cs<<31)|(cs>>1);
 	
 	Uint32 teamsCs=0;
 	for (int i=0; i<session.numberOfTeam; i++)
 	{
-		teamsCs^=teams[i]->checkSum(checkSumsList, checkSumsListForBuildings, checkSumsListForUnits);
+		teamsCs^=teams[i]->checkSum(checkSumsVector, checkSumsVectorForBuildings, checkSumsVectorForUnits);
 		teamsCs=(teamsCs<<31)|(teamsCs>>1);
 		cs=(cs<<31)|(cs>>1);
 	}
 	cs^=teamsCs;
-	if (checkSumsList)
-		checkSumsList->push_back(teamsCs);// [1+t*20]
+	if (checkSumsVector)
+		checkSumsVector->push_back(teamsCs);// [1+t*20]
 	
 	cs=(cs<<31)|(cs>>1);
 	
 	Uint32 playersCs=0;
 	for (int i=0; i<session.numberOfPlayer; i++)
 	{
-		playersCs^=players[i]->checkSum(checkSumsList);
+		playersCs^=players[i]->checkSum(checkSumsVector);
 		playersCs=(playersCs<<31)|(playersCs>>1);
 		cs=(cs<<31)|(cs>>1);
 	}
 	cs^=playersCs;
-	if (checkSumsList)
-		checkSumsList->push_back(playersCs);// [2+t*20+p*2]
+	if (checkSumsVector)
+		checkSumsVector->push_back(playersCs);// [2+t*20+p*2]
 	
 	cs=(cs<<31)|(cs>>1);
 	
@@ -2710,8 +2740,8 @@ Uint32 Game::checkSum(std::list<Uint32> *checkSumsList, std::list<Uint32> *check
 		}
 	Uint32 mapCs=map.checkSum(heavy);
 	cs^=mapCs;
-	if (checkSumsList)
-		checkSumsList->push_back(mapCs);// [3+t*20+p*2]
+	if (checkSumsVector)
+		checkSumsVector->push_back(mapCs);// [3+t*20+p*2]
 	
 	cs=(cs<<31)|(cs>>1);
 
@@ -2720,15 +2750,15 @@ Uint32 Game::checkSum(std::list<Uint32> *checkSumsList, std::list<Uint32> *check
 	syncRandCs^=getSyncRandSeedB();
 	syncRandCs^=getSyncRandSeedC();
 	cs^=syncRandCs;
-	if (checkSumsList)
-		checkSumsList->push_back(syncRandCs);// [4+t*20+p*2]
+	if (checkSumsVector)
+		checkSumsVector->push_back(syncRandCs);// [4+t*20+p*2]
 
 	cs=(cs<<31)|(cs>>1);
 	
 	Uint32 scriptCs=script.checkSum();
 	cs^=scriptCs;
-	if (checkSumsList)
-		checkSumsList->push_back(scriptCs);// [5+t*20+p*2]
+	if (checkSumsVector)
+		checkSumsVector->push_back(scriptCs);// [5+t*20+p*2]
 	
 	return cs;
 }
