@@ -21,6 +21,7 @@
 
 #include <Toolkit.h>
 #include <GAG.h>
+#include <GUIBase.h>
 
 #include "FileManager.h"
 #include "GlobalContainer.h"
@@ -32,6 +33,7 @@
 #include "VoiceRecorder.h"
 #include "IntBuildingType.h"
 #include "Race.h"
+#include "Glob2Screen.h"
 
 // version related stuff
 #ifdef HAVE_CONFIG_H
@@ -88,6 +90,9 @@ GlobalContainer::GlobalContainer(void)
 
 GlobalContainer::~GlobalContainer(void)
 {
+	// unlink GUI style
+	Style::style = &defaultStyle;
+	
 	// close sound
 	if (mix)
 		delete mix;
@@ -431,8 +436,11 @@ void GlobalContainer::load(void)
 		Toolkit::loadFont("data/fonts/sans.ttf", 14, "standard");
 		Toolkit::loadFont("data/fonts/sans.ttf", 10, "little");
 		menuFont = Toolkit::getFont("menu");
+		menuFont->setStyle(Font::Style(Font::STYLE_NORMAL, GAGGUI::ColorTheme::frontColor));
 		standardFont = Toolkit::getFont("standard");
+		standardFont->setStyle(Font::Style(Font::STYLE_NORMAL, GAGGUI::ColorTheme::frontColor));
 		littleFont = Toolkit::getFont("little");
+		littleFont->setStyle(Font::Style(Font::STYLE_NORMAL, GAGGUI::ColorTheme::frontColor));
 
 		updateLoadProgressBar(45);
 		// load terrain data
@@ -451,7 +459,8 @@ void GlobalContainer::load(void)
 		ressources = Toolkit::getSprite("data/gfx/ressource");
 		ressourceMini = Toolkit::getSprite("data/gfx/ressourcemini");
 		bullet = Toolkit::getSprite("data/gfx/bullet");
-		bulletExplosion = Toolkit::getSprite("data/gfx/explosion"); 
+		bulletExplosion = Toolkit::getSprite("data/gfx/explosion");
+		deathAnimation = Toolkit::getSprite("data/gfx/death"); 
 
 		updateLoadProgressBar(70);
 		// load units
@@ -463,8 +472,15 @@ void GlobalContainer::load(void)
 		gamegui = Toolkit::getSprite("data/gfx/gamegui");
 		brush = Toolkit::getSprite("data/gfx/brush");
 		magiceffect = Toolkit::getSprite("data/gfx/magiceffect");
+		Style::style = &style;
 
 		updateLoadProgressBar(100);
 		destroyProgressBar();
 	}
 };
+
+Uint32 GlobalContainer::getConfigCheckSum()
+{
+	// TODO: add the units config
+	return buildingsTypes.checkSum() + ressourcesTypes.checkSum() + Race::checkSumDefault();
+}
