@@ -26,6 +26,7 @@
 #include "Header.h"
 #include "RessourcesTypes.h"
 #include "Settings.h"
+#include "Glob2Style.h"
 
 namespace GAGCore
 {
@@ -81,6 +82,7 @@ public:
 	Sprite *ressourceMini;
 	Sprite *bullet;
 	Sprite *bulletExplosion;
+	Sprite *deathAnimation;
 	Sprite *units;
 	Sprite *unitmini;
 	Sprite *gamegui;
@@ -90,6 +92,8 @@ public:
 	Font *menuFont;
 	Font *standardFont;
 	Font *littleFont;
+	
+	Glob2Style style;
 
 	Settings settings;
 
@@ -105,6 +109,9 @@ public:
 	char hostServerPassWord[32];
 	//! hostname for YOG, can be set by cmd line to override default
 	std::string yogHostName;
+	
+public:
+	Uint32 getConfigCheckSum();
 };
 
 extern GlobalContainer *globalContainer;
