@@ -93,6 +93,10 @@ bool IRC::disconnect(void)
 
 void IRC::forceDisconnect(void)
 {
+	char command[IRC_MESSAGE_SIZE];
+	snprintf(command, IRC_MESSAGE_SIZE, "QUIT :Globulation2");
+	sendString(command);
+
 	if (socket)
 	{
 		SDLNet_TCP_Close(socket);
@@ -277,7 +281,7 @@ void IRC::interpreteIRCMessage(const char *message)
 	}
 	else if (strcasecmp(cmd, "353")==0)
 	{
-		char *diffusion = strtok(NULL, " =");
+		/*char *diffusion =*/ strtok(NULL, " ="); // strtok is used here for its side effects
 		char *chan = strtok(NULL, " :=");
 		char *user;
 		while ((user = strtok(NULL, " :")) != NULL)
