@@ -66,7 +66,7 @@ struct Case
 */
 class Map
 {
-	static const bool verbose = false;
+	static const bool verbose = true;
 public:
 	//! Type of terrain (used for undermap)
 
@@ -157,6 +157,23 @@ public:
 			for (int dy=y; dy<y+h; dy++)
 				setMapBuildingsDiscovered(dx, dy, sharedVision, teams);
 	}
+	
+	//! Make the map at rect (x, y, w, h) explored by unit, i.e. to 255
+	void setMapExploredByUnit(int x, int y, int w, int h, int team)
+	{
+		for (int dx = x; dx < x + w; dx++)
+			for (int dy = y; dy < y + h; dy++)
+				exploredArea[team][((dy & hMask) << wDec) | (dx & wMask)] = 255;
+	}
+	
+	//! Make the map at rect (x, y, w, h) explored by building, i.e. to minimum 2
+	void setMapExploredByBuilding(int x, int y, int w, int h, int team)
+	{
+		for (int dx = x; dx < x + w; dx++)
+			for (int dy = y; dy < y + h; dy++)
+				if (exploredArea[team][((dy & hMask) << wDec) | (dx & wMask)] < 2)
+					exploredArea[team][((dy & hMask) << wDec) | (dx & wMask)] = 2;
+	}
 
 	//! Set all map for all teams to undiscovered state
 	void unsetMapDiscovered(void)
@@ -204,7 +221,7 @@ public:
 	void computeLocalClearArea(int localTeamNo);
 	
 	//! Return the case at a given position
-	Case &getCase(int x, int y)
+	inline Case &getCase(int x, int y)
 	{
 		return cases[((y&hMask)<<wDec)+(x&wMask)];
 	}
@@ -250,6 +267,16 @@ public:
 		return cases[((y&hMask)<<wDec)+(x&wMask)].forbidden;
 	}
 	
+	Uint8 getExplored(int x, int y, int team)
+	{
+		return exploredArea[team][((y&hMask)<<wDec)+(x&wMask)];
+	}
+	
+	Uint8 getGuardAreasGradient(int x, int y, bool canSwim, int team)
+	{
+		return guardAreasGradient[team][canSwim][((y&hMask)<<wDec)+(x&wMask)];
+	}
+	
 	void setTerrain(int x, int y, Uint16 terrain)
 	{
 		cases[((y&hMask)<<wDec)+(x&wMask)].terrain = terrain;
@@ -285,18 +312,18 @@ public:
 
 	bool isRessource(int x, int y)
 	{
-		return cases[((y&hMask)<<wDec)+(x&wMask)].ressource.type != NO_RES_TYPE;
+		return getCase(x, y).ressource.type != NO_RES_TYPE;
 	}
 
-	bool isRessource(int x, int y, int ressourceType)
+	bool isRessourceTakeable(int x, int y, int ressourceType)
 	{
-		const Ressource &ressource = cases[((y&hMask)<<wDec)+(x&wMask)].ressource;
-		return (ressource.type == ressourceType &&  ressource.amount>0);
+		const Ressource &ressource = getCase(x, y).ressource;
+		return (ressource.type == ressourceType && ressource.amount > 0);
 	}
 
-	bool isRessource(int x, int y, bool ressourceTypes[BASIC_COUNT])
+	bool isRessourceTakeable(int x, int y, bool ressourceTypes[BASIC_COUNT])
 	{
-		const Ressource &ressource = cases[((y&hMask)<<wDec)+(x&wMask)].ressource;
+		const Ressource &ressource = getCase(x, y).ressource;
 		return (ressource.type != NO_RES_TYPE
 			&& ressource.amount > 0
 			&& ressource.type < BASIC_COUNT
@@ -305,10 +332,10 @@ public:
 
 	bool isRessource(int x, int y, int *ressourceType)
 	{
-		int rt = getRessource(x, y).type;
-		if (rt == 0xFF)
+		const Ressource &ressource = getCase(x, y).ressource;
+		if (ressource.type == NO_RES_TYPE)
 			return false;
-		*ressourceType = rt;
+		*ressourceType = ressource.type;
 		return true;
 	}
 
@@ -381,8 +408,10 @@ public:
 	
 	//! Transform coordinate from map scale (mx,my) to pixel scale (px,py)
 	void mapCaseToPixelCase(int mx, int my, int *px, int *py) { *px=(mx<<5); *py=(my<<5); }
-	//! Transform coordinate from map (mx,my) to screen (px,py)
+	//! Transform coordinate from map (mx,my) to screen (px,py). Use this one to display a building or an unit to the screen.
 	void mapCaseToDisplayable(int mx, int my, int *px, int *py, int viewportX, int viewportY);
+	//! Transform coordinate from map (mx,my) to screen (px,py). Use this one to display a pathline to the screen.
+	void mapCaseToDisplayableVector(int mx, int my, int *px, int *py, int viewportX, int viewportY, int screenW, int screenH);
 	//! Transform coordinate from screen (mx,my) to map (px,py) for standard grid aligned object (buildings, ressources, units)
 	void displayToMapCaseAligned(int mx, int my, int *px, int *py, int viewportX, int viewportY);
 	//! Transform coordinate from screen (mx,my) to map (px,py) for standard grid unaligned object (terrain)
@@ -406,10 +435,14 @@ public:
 		return gradient[((y&hMask)<<wDec)+(x&wMask)];
 	}
 	
-	void updateGlobalGradientSmall(Uint8 *gradient);
-	void updateGlobalGradientBig(Uint8 *gradient);
-	void updateGlobalGradient(Uint8 *gradient);
+	void updateGlobalGradientSlow(Uint8 *gradient);
+	template<typename Tint> void updateGlobalGradientSlow(Uint8 *gradient);
+	template<typename Tint> void updateGlobalGradient(Uint8 *gradient, Tint *listedAddr, size_t listCountWrite);
+	//void updateGlobalGradientSmall(Uint8 *gradient);
+	//void updateGlobalGradientBig(Uint8 *gradient);
+	//void updateGlobalGradient(Uint8 *gradient);
 	void updateRessourcesGradient(int teamNumber, Uint8 ressourceType, bool canSwim);
+	template<typename Tint> void updateRessourcesGradient(int teamNumber, Uint8 ressourceType, bool canSwim);
 	bool directionFromMinigrad(Uint8 miniGrad[25], int *dx, int *dy, const bool strict, bool verbose);
 	bool directionByMinigrad(Uint32 teamMask, bool canSwim, int x, int y, int *dx, int *dy, Uint8 *gradient, bool strict, bool verbose);
 	bool directionByMinigrad(Uint32 teamMask, bool canSwim, int x, int y, int bx, int by, int *dx, int *dy, Uint8 localGradient[1024], bool strict, bool verbose);
@@ -418,6 +451,7 @@ public:
 	
 	void updateLocalGradient(Building *building, bool canSwim); //The 32*32 gradient
 	void updateGlobalGradient(Building *building, bool canSwim); //The full-sized gradient
+	template<typename Tint> void updateGlobalGradient(Building *building, bool canSwim);
 	//!A special gradient for clearing flags. Returns false if there is nothing to clear.
 	bool updateLocalRessources(Building *building, bool canSwim); 
 	void expandLocalGradient(Uint8 *gradient);
@@ -433,13 +467,18 @@ public:
 	bool pathfindGuardArea(int teamNumber, bool canSwim, int x, int y, int *dx, int *dy);
 	//! Update the forbidden gradient, 
 	void updateForbiddenGradient(int teamNumber, bool canSwim);
+	template<typename Tint> void updateForbiddenGradient(int teamNumber, bool canSwim);
 	void updateForbiddenGradient(int teamNumber);
 	void updateForbiddenGradient();
 	//! Update the guard area gradient
 	void updateGuardAreasGradient(int teamNumber, bool canSwim);
+	template<typename Tint> void updateGuardAreasGradient(int teamNumber, bool canSwim);
 	void updateGuardAreasGradient(int teamNumber);
 	void updateGuardAreasGradient();
 	
+	void initExploredArea(int teamNumber);
+	void updateExploredArea(int teamNumber);
+	
 protected:
 	// computationals pathfinding statistics:
 	int ressourceAvailableCount[16][MAX_RESSOURCES];
@@ -557,6 +596,11 @@ public:
 	//[int team][bool unitCanSwim][int mapX][int mapY]
 	Uint8 *guardAreasGradient[32][2];
 	
+	// Used to guide explorers
+	//[int team]
+	// 0=unexplored, 255=just explored
+	Uint8 *exploredArea[32];
+	
 protected:
 	//Used for scheduling computation time.
 	bool gradientUpdated[32][MAX_NB_RESSOURCES][2];
@@ -582,13 +626,14 @@ public:
 	void controlSand(void);
 	void smoothRessources(int times);
 	bool makeRandomMap(MapGenerationDescriptor &descriptor);
-	void addRessourcesRandomMap(MapGenerationDescriptor &descriptor);
-	bool makeIslandsMap(MapGenerationDescriptor &descriptor);
-	void addRessourcesIslandsMap(MapGenerationDescriptor &descriptor);
+	bool oldMakeRandomMap(MapGenerationDescriptor &descriptor);
+	void oldAddRessourcesRandomMap(MapGenerationDescriptor &descriptor);
+	bool oldMakeIslandsMap(MapGenerationDescriptor &descriptor);
+	void oldAddRessourcesIslandsMap(MapGenerationDescriptor &descriptor);
 
 protected:
 	FILE *logFile;
+	Uint32 incRessourceLog[16];
 };
 
 #endif
- 
