@@ -40,7 +40,7 @@ public:
 	int getDataLength() {return DATA_SIZE; }
 	
 	void save(GAGCore::OutputStream *stream);
-	bool load(GAGCore::InputStream *stream);
+	bool load(GAGCore::InputStream *stream, Sint32 versionMinor);
 	Uint32 checkSum();
 	void saveSyncronization(void);
 	void loadSyncronization(void);
@@ -52,16 +52,22 @@ public:
 	{
 		eNONE=-1,
 		eUNIFORM=0,
-		eRANDOM=1,
-		eISLANDS=2
+		eSWAMP=1,
+		eRIVER=2,
+		eISLANDS=3,
+		eCRATERLAKES=4,
+		eOLDRANDOM=5,
+		eOLDISLANDS=6
 	};
+
 	Methode methode;
 	
 	Sint32 wDec, hDec;
 	
-	Sint32 waterRatio, sandRatio, grassRatio;
+	Sint32 waterRatio, sandRatio, grassRatio, desertRatio, wheatRatio,
+		woodRatio, algaeRatio, stoneRatio, riverDiameter, craterDensity, extraIslands;
+	Sint32 oldIslandSize, oldBeach;
 	Sint32 smooth;
-	Sint32 islandsSize, beach;
 	Sint32 ressource[MAX_NB_RESSOURCES];
 
 	Sint32 nbTeams, nbWorkers;
@@ -72,12 +78,11 @@ public:
 	Sint32 bootX[32];
 	Sint32 bootY[32];
 public:
-	enum {DATA_SIZE=60+MAX_NB_RESSOURCES*4};
+	enum {DATA_SIZE=84+MAX_NB_RESSOURCES*4};
 protected:
 	//! Serialized form of MapGenerationDescriptor
 	Uint8 data[DATA_SIZE];
 };
 
 
-#endif 
- 
+#endif
