@@ -26,7 +26,9 @@
 #include "MapGenerationDescriptor.h"
 #include "Unit.h"
 #include "Utilities.h"
+#include "HeightMapGenerator.h"
 
+///generates a map that is of one type only
 void Map::makeHomogenMap(TerrainType terrainType)
 {
 	for (int y=0; y<h; y++)
@@ -112,7 +114,7 @@ void simulateRandomMap(int smooth, double baseWater, double baseSand, double bas
 	int grassRatio=(int)(baseGrass*((double)totalRatio));
 	totalRatio=waterRatio+sandRatio+grassRatio;
 	
-	// First, we create a fully random patchwork:
+	/// First, we create a fully random patchwork:
 	for (int y=0; y<h; y++)
 		for (int x=0; x<w; x++)
 		{
@@ -309,7 +311,7 @@ void fastSimulateRandomMap(int smooth, double baseWater, double baseSand, double
 	*finalGrass=medianFinalGrass;
 }
 
-bool Map::makeRandomMap(MapGenerationDescriptor &descriptor)
+bool Map::oldMakeRandomMap(MapGenerationDescriptor &descriptor)
 {
 	int waterRatio=descriptor.waterRatio;
 	int sandRatio =descriptor.sandRatio ;
@@ -775,7 +777,215 @@ bool Map::makeRandomMap(MapGenerationDescriptor &descriptor)
 	return true;
 }
 
-void Map::addRessourcesRandomMap(MapGenerationDescriptor &descriptor)
+/// This random map generator generates a heightfield and then choses levels at wich to draw the line between water, sand and gras
+bool Map::makeRandomMap(MapGenerationDescriptor &descriptor)
+{
+	/// all under waterLevel is water, under sandLevel is beach, under grassLevel is grass and above grasslevel is desert
+	float waterLevel, sandLevel, grassLevel, wheatWoodLevel, algaeLevel;
+	/// to influence the roughnes
+	float smoothingFactor=(float)(descriptor.smooth+4)*3;;
+	/// the proportions requested through the gui can directly be translated into tile counts of the undermap.
+	unsigned int waterTiles, sandTiles, grassTiles, wheatWoodTiles, algaeTiles;
+	/// grass + sand + water + desert as from the gui
+
+	unsigned int totalGSWFromUI=descriptor.waterRatio+descriptor.sandRatio+descriptor.grassRatio+descriptor.desertRatio;
+	HeightMap hm(w,h);
+
+	unsigned int tmpTotal=1+descriptor.waterRatio+descriptor.grassRatio;
+	switch (descriptor.methode)
+	{
+		case MapGenerationDescriptor::eSWAMP:
+			hm.makeSwamp(smoothingFactor);
+			waterTiles=(unsigned int)((float)descriptor.waterRatio*w*h/(float)tmpTotal);
+			sandTiles=0;
+			grassTiles=w*h-waterTiles;
+			break;
+		case MapGenerationDescriptor::eRIVER:
+			hm.makeRiver(descriptor.riverDiameter*(w+h)/2/100,smoothingFactor);
+			waterTiles=(unsigned int)((float)descriptor.waterRatio/(float)totalGSWFromUI*w*h);
+			sandTiles=(unsigned int)((float)descriptor.sandRatio/(float)totalGSWFromUI*w*h);
+			grassTiles =(unsigned int)((float)descriptor.grassRatio /(float)totalGSWFromUI*w*h);
+			break;
+		case MapGenerationDescriptor::eCRATERLAKES:
+			hm.makeCraters(w*h*descriptor.craterDensity/30000, 30, smoothingFactor);
+			waterTiles=(unsigned int)((float)descriptor.waterRatio/(float)totalGSWFromUI*w*h);
+			sandTiles=(unsigned int)((float)descriptor.sandRatio/(float)totalGSWFromUI*w*h);
+			grassTiles =(unsigned int)((float)descriptor.grassRatio /(float)totalGSWFromUI*w*h);
+			break;
+		case MapGenerationDescriptor::eISLANDS:
+			hm.makeIslands(descriptor.nbTeams+descriptor.extraIslands, smoothingFactor);
+			waterTiles=(unsigned int)((float)descriptor.waterRatio/(float)totalGSWFromUI*w*h);
+			sandTiles=(unsigned int)((float)descriptor.sandRatio/(float)totalGSWFromUI*w*h);
+			grassTiles =(unsigned int)((float)descriptor.grassRatio /(float)totalGSWFromUI*w*h);
+			break;
+		default: assert(false);
+			break;
+	}
+	/// wheat/wood needs ground to stand on and water. So:
+	wheatWoodTiles=waterTiles<grassTiles?waterTiles/2:grassTiles/2;
+	algaeTiles=waterTiles/6;
+
+	/// histogram[i] collects the count of all terrain levels == i
+	int histogram[2048];
+	memset(histogram, 0, 2048*sizeof(int));
+
+	for (int i=0; i<w*h; i++)
+	{
+		histogram[hm.uiLevel(i,2048)]++;
+	}
+/* 	for (int i=0; i<2048; i++)
+	{
+		for (int j=0; j<2048*histogram[i]/w/h; j++)
+			std::cout << "#";
+		if(i%10==0)
+			std::cout << "\n";
+	} */
+	
+	unsigned int accumulatedHistogram=0;
+	int i=0;
+	waterLevel=0;
+	sandLevel=0;
+	grassLevel=0;
+	wheatWoodLevel=0;
+	algaeLevel=0;
+	while ((waterLevel==0) & (i<2048))
+	{
+		accumulatedHistogram+=histogram[i++];
+		if (algaeLevel==0 && accumulatedHistogram >= algaeTiles)
+			algaeLevel = (float)(i-1)/2048.0;
+		if (accumulatedHistogram >= waterTiles)
+			waterLevel = (float)(i-1)/2048.0;
+	}
+	while ((sandLevel==0) && (i<2048))
+	{
+		accumulatedHistogram+=histogram[i++];
+		if (accumulatedHistogram >= waterTiles+sandTiles)
+			sandLevel = (float)(i-1)/2048.0;
+	}
+	while ((grassLevel==0) && (i<2048))
+	{
+		accumulatedHistogram+=histogram[i++];
+		if (wheatWoodLevel==0 && accumulatedHistogram >= waterTiles+sandTiles+wheatWoodTiles)
+			wheatWoodLevel = (float)(i-1)/2048.0;
+		if (accumulatedHistogram >= waterTiles+sandTiles+grassTiles)
+			grassLevel = (float)(i-1)/2048.0;
+	}
+
+	for (int i=0; i<w*h; i++)
+	{
+			if (hm(i)<waterLevel)
+				undermap[i]=WATER;
+			else if (hm(i)<sandLevel)
+				undermap[i]=SAND;
+			else if (hm(i)<grassLevel)
+				undermap[i]=GRASS;
+			else
+				undermap[i]=SAND;
+	}
+	controlSand();
+	//std::cout << "writing heightMap (raw) to your glob folder\n";
+	//hm.mapOutput("heightMap");
+
+	//Now, we have to find suitable places for teams:
+	int nbTeams=descriptor.nbTeams;
+	int minDistSquare=(int)((double)w*h/(double)nbTeams/5);
+	std::cout << "minDistSquare=" << minDistSquare << " (" << sqrt((double)minDistSquare) << ").\n";
+	if (minDistSquare<=0)
+	{
+		std::cout << "debugoutput 1\n";
+		return false;
+	}
+	assert(minDistSquare>0);
+	int* bootX=descriptor.bootX;
+	int* bootY=descriptor.bootY;
+	
+	//TODO: First pass to find the number of available places.
+	for (int team=0; team<nbTeams; team++)
+	{
+		int maxSurface=0;
+		int maxX=0;
+		int maxY=0;
+		for (int y=0; y<h; y++)
+		{
+			int width=0;
+			int startx=0;
+			for (int x=0; x<w; x++)
+			{
+				int a=undermap[y*w+x];
+				if (a==GRASS)
+					width++;
+				else
+				{
+					if (width>7)
+					{
+						int centerx=((x+startx)>>1);
+						int top, bot;
+						for (top=0; top<h; top++)
+							if (getUMTerrain(centerx, y-top)!=GRASS)
+								break;
+						for (bot=0; bot<h; bot++)
+							if (getUMTerrain(centerx, y+bot)!=GRASS)
+								break;
+						int height=top+bot-1;
+						int surface=height*width;
+						assert(surface>0);
+						
+						int centery=y+((bot-top)>>1);
+						bool farEnough=true;
+						for (int ti=0; ti<team; ti++)
+							if (warpDistSquare(centerx, centery, bootX[ti], bootY[ti])<minDistSquare)
+							{
+								farEnough=false;
+								break;
+							}
+						
+						if (surface>maxSurface && farEnough)
+						{
+							maxSurface=surface;
+							maxX=centerx;
+							maxY=centery;
+						}
+					}
+					width=0;
+					startx=x;
+				}
+			}
+		}
+		
+		if (maxSurface<=0)
+		{
+			std::cout << "debugoutput 2\n";
+			return false;
+		}
+		assert(maxSurface);
+		bootX[team]=maxX;
+		bootY[team]=maxY;
+	}
+	
+	controlSand();
+	regenerateMap(0, 0, w, h);
+	
+	for(int x=0; x<w; x++)
+	{
+		for (int y=0; y<h; y++)
+		{
+		if(hm(x+w*y)<algaeLevel)
+			setRessource(x,y, ALGA, 1);
+		else if(hm(x+w*y)<wheatWoodLevel)
+			if(hm((x+w/2)%w+w*y)<hm((x+w/2+1)%w+w*y))
+				setRessource(x,y,CORN,1);
+			else
+				setRessource(x,y,WOOD,1);
+		}
+	}
+
+	if (verbose)
+		printf("makeRandomMap::success\n");
+	return true;
+}
+
+
+void Map::oldAddRessourcesRandomMap(MapGenerationDescriptor &descriptor)
 {
 	int *bootX=descriptor.bootX;
 	int *bootY=descriptor.bootY;
@@ -936,7 +1146,7 @@ void Map::addRessourcesRandomMap(MapGenerationDescriptor &descriptor)
 	smoothRessources(maxAmount*3);
 }
 
-bool Map::makeIslandsMap(MapGenerationDescriptor &descriptor)
+bool Map::oldMakeIslandsMap(MapGenerationDescriptor &descriptor)
 {
 	// First, fill with water:
 	for (int y=0; y<h; y++)
@@ -947,7 +1157,7 @@ bool Map::makeIslandsMap(MapGenerationDescriptor &descriptor)
 	int* bootX=descriptor.bootX;
 	int* bootY=descriptor.bootY;
 	int nbIslands=descriptor.nbTeams;
-	int islandsSize=(int)(((w+h)*descriptor.islandsSize)/(400.0*sqrt((double)nbIslands)));
+	int islandsSize=(int)(((w+h)*descriptor.oldIslandSize)/(400.0*sqrt((double)nbIslands)));
 	if (islandsSize<8)
 		islandsSize=8;
 	int minDistSquare=(w*h)/nbIslands;
@@ -1209,7 +1419,7 @@ bool Map::makeIslandsMap(MapGenerationDescriptor &descriptor)
 	controlSand();
 	
 	// Five, add some sand
-	for (int s=0; s<descriptor.beach; s++)
+	for (int s=0; s<descriptor.oldBeach; s++)
 		for (int dy=0; dy<4; dy++)
 			for (int dx=0; dx<4; dx++)
 				for (int y=dy; y<h; y+=4)
@@ -1301,12 +1511,12 @@ bool Map::makeIslandsMap(MapGenerationDescriptor &descriptor)
 	return true;
 }
 
-void Map::addRessourcesIslandsMap(MapGenerationDescriptor &descriptor)
+void Map::oldAddRessourcesIslandsMap(MapGenerationDescriptor &descriptor)
 {
 	int *bootX=descriptor.bootX;
 	int *bootY=descriptor.bootY;
 	
-	int islandsSize=(int)(((w+h)*descriptor.islandsSize)/(400.0*sqrt((double)descriptor.nbTeams)));
+	int islandsSize=(int)(((w+h)*descriptor.oldIslandSize)/(400.0*sqrt((double)descriptor.nbTeams)));
 	if (islandsSize<8)
 		islandsSize=8;
 	// let's add ressources...
@@ -1381,13 +1591,13 @@ void Map::addRessourcesIslandsMap(MapGenerationDescriptor &descriptor)
 	this->smoothRessources(smoothRessources*2);
 }
 
-bool Game::makeIslandsMap(MapGenerationDescriptor &descriptor)
+bool Game::oldMakeIslandsMap(MapGenerationDescriptor &descriptor)
 {
 	for (int s=0; s<descriptor.nbTeams; s++)
 	{
 		if (session.numberOfTeam<=s)
 			addTeam();
-		int squareSize=5+descriptor.islandsSize/10;
+		int squareSize=5+descriptor.oldIslandSize/10;
 		map.setUMatPos(descriptor.bootX[s]+2, descriptor.bootY[s]+0, GRASS, squareSize);
 		map.setUMatPos(descriptor.bootX[s]+2, descriptor.bootY[s]+2, GRASS, squareSize);
 		
@@ -1411,7 +1621,7 @@ bool Game::makeIslandsMap(MapGenerationDescriptor &descriptor)
 			}
 		teams[s]->createLists();
 	}
-	map.smoothRessources(descriptor.islandsSize/10);
+	map.smoothRessources(descriptor.oldIslandSize/10);
 	return true;
 }
 
@@ -1464,20 +1674,28 @@ bool Game::generateMap(MapGenerationDescriptor &descriptor)
 			map.makeHomogenMap(descriptor.terrainType);
 			addTeam();
 		break;
-		case MapGenerationDescriptor::eRANDOM:
+		case MapGenerationDescriptor::eSWAMP:
+		case MapGenerationDescriptor::eISLANDS:
+		case MapGenerationDescriptor::eRIVER:
+		case MapGenerationDescriptor::eCRATERLAKES:
 			if (!map.makeRandomMap(descriptor))
 				return false;
-			map.addRessourcesRandomMap(descriptor);
 			if (!makeRandomMap(descriptor))
 				return false;
-		break;
-		case MapGenerationDescriptor::eISLANDS:
-			if (!map.makeIslandsMap(descriptor))
+			break;
+		case MapGenerationDescriptor::eOLDRANDOM:
+			if (!map.oldMakeRandomMap(descriptor))
 				return false;
-			map.addRessourcesIslandsMap(descriptor);
-			if (!makeIslandsMap(descriptor))
+			if (!makeRandomMap(descriptor))
 				return false;
-		break;
+			break;
+		case MapGenerationDescriptor::eOLDISLANDS:
+			if (!map.oldMakeIslandsMap(descriptor))
+				return false;
+			if (!oldMakeIslandsMap(descriptor))
+				return false;
+			break;
+						
 		default:
 			assert(false);
 	}
@@ -1488,8 +1706,3 @@ bool Game::generateMap(MapGenerationDescriptor &descriptor)
 		printf(".... map generated.\n");
 	return true;
 }
-/*
-WATER=0,
-SAND=1,
-GRASS=2
-*/
