@@ -84,6 +84,7 @@ Unit::Unit(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, int level)
 
 	targetX=0;
 	targetY=0;
+	validTarget=false;
 	magicActionTimeout=0;
 
 	// trigger parameters
@@ -159,6 +160,10 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	action = (Abilities)stream->readUint32("action");
 	targetX = (Sint32)stream->readSint32("targetX");
 	targetY = (Sint32)stream->readSint32("targetY");
+	if (versionMinor >= 46)
+		validTarget = (bool)stream->readSint32("validTarget");
+	else
+		validTarget = false;
 	
 	if (versionMinor >= 41)
 		magicActionTimeout = stream->readSint32("magicActionTimeout");
@@ -256,6 +261,7 @@ void Unit::save(GAGCore::OutputStream *stream)
 	stream->writeUint32((Uint32)action, "action");
 	stream->writeSint32(targetX, "targetX");
 	stream->writeSint32(targetY, "targetY");
+	stream->writeSint32(validTarget, "validTarget");
 	stream->writeSint32(magicActionTimeout, "magicActionTimeout");
 
 	// trigger parameters
@@ -364,6 +370,7 @@ void Unit::subscriptionSuccess(void)
 			assert(targetBuilding==attachedBuilding);
 			targetX=targetBuilding->getMidX();
 			targetY=targetBuilding->getMidY();
+			validTarget=true;
 		}
 		break;
 		case MED_FREE:
@@ -376,6 +383,7 @@ void Unit::subscriptionSuccess(void)
 					assert(targetBuilding==attachedBuilding);
 					targetX=attachedBuilding->getMidX();
 					targetY=attachedBuilding->getMidY();
+					validTarget=true;
 				}
 				break;
 				case ACT_UPGRADING:
@@ -384,6 +392,7 @@ void Unit::subscriptionSuccess(void)
 					assert(targetBuilding==attachedBuilding);
 					targetX=targetBuilding->getMidX();
 					targetY=targetBuilding->getMidY();
+					validTarget=true;
 				}
 				break;
 				case ACT_FILLING:
@@ -395,6 +404,7 @@ void Unit::subscriptionSuccess(void)
 						targetBuilding=attachedBuilding;
 						targetX=targetBuilding->getMidX();
 						targetY=targetBuilding->getMidY();
+						validTarget=true;
 					}
 					else if (ownExchangeBuilding)
 					{
@@ -410,6 +420,7 @@ void Unit::subscriptionSuccess(void)
 								targetBuilding=foreingExchangeBuilding;
 								targetX=targetBuilding->getMidX();
 								targetY=targetBuilding->getMidY();
+								validTarget=true;
 							}
 							else
 							{
@@ -417,6 +428,7 @@ void Unit::subscriptionSuccess(void)
 								targetBuilding=ownExchangeBuilding;
 								targetX=targetBuilding->getMidX();
 								targetY=targetBuilding->getMidY();
+								validTarget=true;
 							}
 						}
 						else
@@ -432,6 +444,7 @@ void Unit::subscriptionSuccess(void)
 								targetBuilding=attachedBuilding;
 								targetX=targetBuilding->getMidX();
 								targetY=targetBuilding->getMidY();
+								validTarget=true;
 							}
 							else
 							{
@@ -440,6 +453,7 @@ void Unit::subscriptionSuccess(void)
 								targetBuilding=ownExchangeBuilding;
 								targetX=targetBuilding->getMidX();
 								targetY=targetBuilding->getMidY();
+								validTarget=true;
 							}
 						}
 					}
@@ -447,14 +461,16 @@ void Unit::subscriptionSuccess(void)
 					{
 						displacement=DIS_GOING_TO_RESSOURCE;
 						targetBuilding=NULL;
-						bool rv=owner->map->ressourceAvailable(owner->teamNumber, destinationPurprose, performance[SWIM], posX, posY, &targetX, &targetY, NULL);
-						fprintf(logFile, "[%d] raa targetXY=(%d, %d)=%d\n", gid, targetX, targetY, rv);
+						owner->map->ressourceAvailable(owner->teamNumber, destinationPurprose, performance[SWIM], posX, posY, &targetX, &targetY, NULL);
+						validTarget=true;
+						//fprintf(logFile, "[%d] raa targetXY=(%d, %d)=%d\n", gid, targetX, targetY, rv);
 					}
 				}
 				break;
 				case ACT_RANDOM :
 				{
 					displacement=DIS_RANDOM;
+					validTarget=false;
 				}
 				break;
 				default:
@@ -481,7 +497,7 @@ void Unit::syncStep(void)
 			if (degats<=0)
 				degats=1;
 			enemy->hp-=degats;
-			enemy->owner->setEvent(posX+dx, posY+dy, Team::UNIT_UNDER_ATTACK_EVENT, enemyGUID);
+			enemy->owner->setEvent(posX+dx, posY+dy, Team::UNIT_UNDER_ATTACK_EVENT, enemyGUID, enemyTeam);
 			incrementExperience(degats);
 		}
 		else
@@ -496,7 +512,7 @@ void Unit::syncStep(void)
 				if (degats<=0)
 					degats=1;
 				enemy->hp-=degats;
-				enemy->owner->setEvent(posX+dx, posY+dy, Team::BUILDING_UNDER_ATTACK_EVENT, enemyGBID);
+				enemy->owner->setEvent(posX+dx, posY+dy, Team::BUILDING_UNDER_ATTACK_EVENT, enemyGBID, enemyTeam);
 				if (enemy->hp<0)
 					enemy->kill();
 				incrementExperience(degats);
@@ -524,11 +540,13 @@ void Unit::syncStep(void)
 		{
 			owner->map->setMapDiscovered(posX-3, posY-3, 7, 7, owner->sharedVisionOther);
 			owner->map->setMapBuildingsDiscovered(posX-3, posY-3, 7, 7, owner->sharedVisionOther, owner->game->teams);
+			owner->map->setMapExploredByUnit(posX-3, posY-3, 7, 7, owner->teamNumber);
 		}
 		else
 		{
 			owner->map->setMapDiscovered(posX-1, posY-1, 3, 3, owner->sharedVisionOther);
 			owner->map->setMapBuildingsDiscovered(posX-1, posY-1, 3, 3, owner->sharedVisionOther, owner->game->teams);
+			owner->map->setMapExploredByUnit(posX-1, posY-1, 3, 3, owner->teamNumber);
 		}
 	}
 	
@@ -581,6 +599,7 @@ void Unit::standardRandomActivity()
 	foreingExchangeBuilding=NULL;
 	activity=Unit::ACT_RANDOM;
 	displacement=Unit::DIS_RANDOM;
+	validTarget=false;
 	needToRecheckMedical=true;
 	subscribed=false;
 }
@@ -613,6 +632,7 @@ void Unit::stopAttachedForBuilding(bool goingInside)
 	
 	activity=ACT_RANDOM;
 	displacement=DIS_RANDOM;
+	validTarget=false;
 	
 	attachedBuilding->unitsWorking.remove(this);
 	attachedBuilding->unitsWorkingSubscribe.remove(this);
@@ -670,11 +690,11 @@ void Unit::handleMagic(void)
 						if (owner->enemies & targetTeamMask)
 						{
 							Unit *enemyUnit = teams[targetTeam]->myUnits[targetID];
-							Sint32 damage = attackForce + experienceLevel - enemyUnit->performance[ARMOR];
+							Sint32 damage = attackForce + experienceLevel - enemyUnit->getRealArmor();
 							if (damage > 0)
 							{
 								enemyUnit->hp -= damage;
-								enemyUnit->owner->setEvent(xi, yi, Team::UNIT_UNDER_ATTACK_EVENT, targetGUID);
+								enemyUnit->owner->setEvent(xi, yi, Team::UNIT_UNDER_ATTACK_EVENT, targetGUID, targetTeam);
 								incrementExperience(damage);
 								magicActionAnimation = MAGIC_ACTION_ANIMATION_FRAME_COUNT;
 								hasUsedMagicAction = true;
@@ -699,7 +719,7 @@ void Unit::handleMagic(void)
 							if (damage > 0)
 							{
 								enemyBuilding->hp -= damage;
-								enemyBuilding->owner->setEvent(xi, yi, Team::BUILDING_UNDER_ATTACK_EVENT, targetGBID);
+								enemyBuilding->owner->setEvent(xi, yi, Team::BUILDING_UNDER_ATTACK_EVENT, targetGBID, targetTeam);
 								if (enemyBuilding->hp <= 0)
 									enemyBuilding->kill();
 								incrementExperience(damage);
@@ -742,6 +762,7 @@ void Unit::handleMedical(void)
 		
 		if (!isDead)
 		{
+			// disconnect from building
 			if (attachedBuilding)
 			{
 				assert((displacement!=DIS_ENTERING_BUILDING) && (displacement!=DIS_INSIDE) && (displacement!=DIS_EXITING_BUILDING));
@@ -758,13 +779,18 @@ void Unit::handleMedical(void)
 			}
 			
 			activity=ACT_RANDOM;
+			validTarget=false;
 			
+			// remove from map
 			if (performance[FLY])
 				owner->map->setAirUnit(posX, posY, NOGUID);
 			else
 				owner->map->setGroundUnit(posX, posY, NOGUID);
+				
+			// generate death animation
+			owner->map->getSector(posX, posY)->deathAnimations.push_back(new UnitDeathAnimation(posX, posY, owner));
 		}
-		isDead=true;
+		isDead = true;
 	}
 }
 
@@ -931,6 +957,7 @@ void Unit::handleActivity(void)
 						printf("guid=(%d) Going to heal building\n", gid);
 					targetX=attachedBuilding->getMidX();
 					targetY=attachedBuilding->getMidY();
+					validTarget=true;
 					b->unitsInsideSubscribe.push_front(this);
 					subscribed=true;
 					if (b->subscribeForInside!=1)
@@ -981,9 +1008,9 @@ void Unit::handleActivity(void)
 					// Unit conversion code
 					
 					// Send events and keep track of number of unit converted
-					currentTeam->setEvent(posX, posY, Team::UNIT_CONVERTED_LOST, typeNum);
+					currentTeam->setEvent(posX, posY, Team::UNIT_CONVERTED_LOST, typeNum, targetTeam->teamNumber);
 					currentTeam->unitConversionLost++;
-					targetTeam->setEvent(posX, posY, Team::UNIT_CONVERTED_ACQUIERED, typeNum);
+					targetTeam->setEvent(posX, posY, Team::UNIT_CONVERTED_ACQUIERED, typeNum, currentTeam->teamNumber);
 					targetTeam->unitConversionGained++;
 					
 					// Find free slot in other team
@@ -1080,6 +1107,7 @@ void Unit::handleDisplacement(void)
 		if (verbose)
 			printf("guid=(%d) handleDisplacement() subscribed, then displacement=DIS_RANDOM\n", gid);
 		displacement=DIS_RANDOM;
+		validTarget=false;
 	}
 	else switch (activity)
 	{
@@ -1094,6 +1122,7 @@ void Unit::handleDisplacement(void)
 			}
 			else
 				displacement=DIS_RANDOM;
+			validTarget=false;
 		}
 		break;
 		
@@ -1108,7 +1137,10 @@ void Unit::handleDisplacement(void)
 			if (displacement==DIS_GOING_TO_RESSOURCE)
 			{
 				if (owner->map->doesUnitTouchRessource(this, destinationPurprose, &dx, &dy))
+				{
 					displacement=DIS_HARVESTING;
+					validTarget=false;
+				}
 			}
 			else if (displacement==DIS_HARVESTING)
 			{
@@ -1121,19 +1153,26 @@ void Unit::handleDisplacement(void)
 				
 				targetBuilding=attachedBuilding;
 				if (owner->map->doesUnitTouchBuilding(this, attachedBuilding->gid, &dx, &dy))
+				{
 					displacement=DIS_FILLING_BUILDING;
+					validTarget=false;
+				}
 				else
 				{
 					displacement=DIS_GOING_TO_BUILDING;
 					targetX=targetBuilding->getMidX();
 					targetY=targetBuilding->getMidY();
+					validTarget=true;
 				}
 			}
 			else if (displacement==DIS_GOING_TO_BUILDING)
 			{
 				assert(targetBuilding);
 				if (owner->map->doesUnitTouchBuilding(this, targetBuilding->gid, &dx, &dy))
+				{
 					displacement=DIS_FILLING_BUILDING;
+					validTarget=false;
+				}
 			}
 			else if (displacement==DIS_FILLING_BUILDING)
 			{
@@ -1185,6 +1224,7 @@ void Unit::handleDisplacement(void)
 						displacement=DIS_GOING_TO_BUILDING;
 						targetX=targetBuilding->getMidX();
 						targetY=targetBuilding->getMidY();
+						validTarget=true;
 						exchangeReady=true;
 						if (verbose)
 							printf("guid=(%d) exchangeReady at foreingExchangeBuilding\n", gid);
@@ -1242,6 +1282,7 @@ void Unit::handleDisplacement(void)
 							displacement=DIS_GOING_TO_BUILDING;
 							targetX=targetBuilding->getMidX();
 							targetY=targetBuilding->getMidY();
+							validTarget=true;
 							exchangeReady=true;
 							if (verbose)
 								printf("guid=(%d) exchangeReady at ownExchangeBuilding\n", gid);
@@ -1276,6 +1317,7 @@ void Unit::handleDisplacement(void)
 						displacement=DIS_GOING_TO_BUILDING;
 						targetX=targetBuilding->getMidX();
 						targetY=targetBuilding->getMidY();
+						validTarget=true;
 						exchangeReady=true;
 						if (verbose)
 							printf("guid=(%d) took a foreign fruit in our exhange building to food\n", gid);
@@ -1296,9 +1338,11 @@ void Unit::handleDisplacement(void)
 						break;
 
 						case Building::NEW_BUILDING:
-						case Building::UPGRADE: 
+						case Building::UPGRADE:
 						{
 							attachedBuilding->hp+=bt->hpInc;
+							if (attachedBuilding->hp > bt->hpMax)
+								attachedBuilding->hp = bt->hpMax;
 						}
 						break;
 
@@ -1326,6 +1370,7 @@ void Unit::handleDisplacement(void)
 							printf("guid=(%d) The building doesn't need me any more.\n", gid);
 						activity=ACT_RANDOM;
 						displacement=DIS_RANDOM;
+						validTarget=false;
 						subscribed=false;
 						assert(needToRecheckMedical);
 					}
@@ -1398,16 +1443,21 @@ void Unit::handleDisplacement(void)
 									displacement=DIS_GOING_TO_BUILDING;
 									targetX=targetBuilding->getMidX();
 									targetY=targetBuilding->getMidY();
+									validTarget=true;
 								}
 								else
 								{
 									int dummyDist;
 									if (owner->map->doesUnitTouchRessource(this, destinationPurprose, &dx, &dy))
+									{
 										displacement=DIS_HARVESTING;
+										validTarget=false;
+									}
 									else if (map->ressourceAvailable(teamNumber, destinationPurprose, canSwim, posX, posY, &targetX, &targetY, &dummyDist))
 									{
 										fprintf(logFile, "[%d] rab targetXY=(%d, %d)\n", gid, targetX, targetY);
 										displacement=DIS_GOING_TO_RESSOURCE;
+										validTarget=true;
 									}
 									else
 									{
@@ -1467,6 +1517,7 @@ void Unit::handleDisplacement(void)
 									targetBuilding=ownExchangeBuilding;
 									targetX=targetBuilding->getMidX();
 									targetY=targetBuilding->getMidY();
+									validTarget=true;
 								}
 								else
 								{
@@ -1492,7 +1543,10 @@ void Unit::handleDisplacement(void)
 				}
 			}
 			else
+			{
 				displacement=DIS_RANDOM;
+				validTarget=false;
+			}
 		}
 		break;
 		
@@ -1503,7 +1557,10 @@ void Unit::handleDisplacement(void)
 			if (displacement==DIS_GOING_TO_BUILDING)
 			{
 				if (owner->map->doesUnitTouchBuilding(this, attachedBuilding->gid, &dx, &dy))
+				{
 					displacement=DIS_ENTERING_BUILDING;
+					validTarget=false;
+				}
 			}
 			else if (displacement==DIS_ENTERING_BUILDING)
 			{
@@ -1515,6 +1572,7 @@ void Unit::handleDisplacement(void)
 				else
 					owner->map->setGroundUnit(posX-dx, posY-dy, NOGUID);
 				displacement=DIS_INSIDE;
+				validTarget=false;
 				
 				if (destinationPurprose==FEED)
 				{
@@ -1540,6 +1598,7 @@ void Unit::handleDisplacement(void)
 				{
 					//printf("Exiting building\n");
 					displacement=DIS_EXITING_BUILDING;
+					validTarget=false;
 
 					if (destinationPurprose==FEED)
 					{
@@ -1556,13 +1615,27 @@ void Unit::handleDisplacement(void)
 					}
 					else
 					{
-						//printf("Ability %d got level %d\n", destinationPurprose, attachedBuilding->type->level+1);
-						assert(canLearn[destinationPurprose]);
-						level[destinationPurprose]=attachedBuilding->type->level+1;
-						UnitType *ut=race->getUnitType(typeNum, level[destinationPurprose]);
-						performance[destinationPurprose]=ut->performance[destinationPurprose];
-
-						//printf("New performance[%d]=%d\n", destinationPurprose, performance[destinationPurprose]);
+						if (attachedBuilding->type->upgradeInParallel)
+						{
+							for (int ability = (int)WALK; ability < (int)ARMOR; ability++)
+								if (canLearn[ability] && attachedBuilding->type->upgrade[ability])
+								{
+									level[ability] = attachedBuilding->type->level + 1;
+									UnitType *ut = race->getUnitType(typeNum, level[ability]);
+									performance[ability] = ut->performance[ability];
+								}
+						}
+						else
+						{
+							//printf("Ability %d got level %d\n", destinationPurprose, attachedBuilding->type->level+1);
+							assert(canLearn[destinationPurprose]);
+							level[destinationPurprose] = attachedBuilding->type->level + 1;
+							UnitType *ut = race->getUnitType(typeNum, level[destinationPurprose]);
+							performance[destinationPurprose] = ut->performance[destinationPurprose];
+							//printf("New performance[%d]=%d\n", destinationPurprose, performance[destinationPurprose]);
+						}
+							
+							
 					}
 				}
 				else
@@ -1575,24 +1648,29 @@ void Unit::handleDisplacement(void)
 				// we want to get out, so we still stay in displacement==DIS_EXITING_BUILDING.
 			}
 			else
+			{
 				displacement=DIS_RANDOM;
+				validTarget=false;
+			}
 		}
 		break;
 
 		case ACT_FLAG:
 		{
 			assert(attachedBuilding);
+			displacement=DIS_GOING_TO_FLAG;
 			targetX=attachedBuilding->posX;
 			targetY=attachedBuilding->posY;
+			validTarget=true;
 			int distance=owner->map->warpDistSquare(targetX, targetY, posX, posY);
 			int usr=attachedBuilding->unitStayRange;
 			int usr2=usr*usr;
 			if (verbose)
 				printf("guid=(%d) ACT_FLAG distance=%d, usr2=%d\n", gid, distance, usr2);
 			
-			displacement=DIS_GOING_TO_FLAG;
 			if (distance<=usr2)
 			{
+				validTarget=false;
 				if (typeNum==WORKER)
 					displacement=DIS_CLEARING_RESSOURCES;
 				else if (typeNum==EXPLORER)
@@ -1614,10 +1692,11 @@ void Unit::handleDisplacement(void)
 							int x=posX+tdx;
 							int y=posY+tdy;
 							if (map->warpDistSquare(x, y, targetX, targetY)<=usr2
-								&& map->isRessource(x, y, attachedBuilding->clearingRessources))
+								&& map->isRessourceTakeable(x, y, attachedBuilding->clearingRessources))
 							{
 								dx=tdx;
 								dy=tdy;
+								validTarget=false;
 								displacement=DIS_CLEARING_RESSOURCES;
 								//movement=MOV_HARVESTING;
 								return;
@@ -1702,71 +1781,63 @@ void Unit::handleMovement(void)
 			}
 			else if ((movement!=MOV_GOING_DXDY)||((syncRand()&0xFF)<0xEF))
 			{
-				int dist[8];
-				int minDist=32;
-				int minDistj=8;
-				for (int j=0; j<8; j++)
+				// "c" is the center of the unit, "x" are the sample spots:
+				// oxoooxo
+				//ooooooooo
+				//xooooooox
+				//ooooooooo
+				//oooocoooo
+				//ooooooooo
+				//xooooooox
+				//ooooooooo
+				// oxoooxo
+				bool found = false;
+				const int dxTab[8] = {-4, -2, +2, +4, +4, +2, -2, -4};
+				const int dyTab[8] = {-2, -4, -4, -2, +2, +4, +4, +2};
+				int tab[8];
+				for (int i = 0; i < 8; i++)
+					tab[i] = owner->map->getExplored(posX + dxTab[i], posY + dyTab[i], owner->teamNumber);
+				//printf("tab ");
+				//for (int i = 0; i < 8; i++)
+				//	printf("%3d; ", tab[i]);
+				//printf("d=%d\n", direction);
+				for (int di = 0; di < 8; di++)
 				{
-					dist[j]=32;
-					// WARNING : the i=4 is linked to the sight range of the explorer.
-					for (int i=4; i<32; i+=4)
+					int d = (di + direction + 4) & 7;
+					if ((tab[d] > 0) && (tab[(d + 1) & 7] == 0) && (tab[(d + 2) & 7] == 0))
 					{
-						int dx, dy;
-						dxdyfromDirection(j, &dx, &dy);
-						if (!owner->map->isMapDiscovered(posX+i*dx, posY+j*dy, owner->sharedVisionOther))
-						{
-							dist[j]=i;
-							break;
-						}
-					}
-					if (dist[j]<minDist)
-					{
-						minDist=dist[j];
-						minDistj=j;
-					}
-					if (verbose)
-						printf("dist[%d]=%d.\n", j, dist[j]);
-				}
-				
-				if (minDist==32)
-				{
-					if (verbose)
-						printf("guid=(%d) I can''t see black.\n", gid);
-					if ((syncRand()&0xFF)<0xEF)
-					{
-						movement=MOV_GOING_DXDY;
-					}
-					else if ((syncRand()&0xFF)<0xEF)
-					{
-						directionFromDxDy();
-						direction=(direction+((syncRand()&1)<<1)+7)&7;
+						direction = (d + 1) & 7;
 						dxdyfromDirection();
-						movement=MOV_GOING_DXDY;
-					}
-					else 
-					{
-						movement=MOV_RANDOM_FLY;
+						movement = MOV_GOING_DXDY;
+						found = true;
+						break;
 					}
 				}
-				else
+				if (!found)
 				{
-					int decj=syncRand()&7;
-					if (verbose)
-						printf("guid=(%d) minDist=%d, minDistj=%d.\n", gid, minDist, minDistj);
-					for (int j=0; j<8; j++)
+					int scoreX = 0;
+					int scoreY = 0;
+					for (int delta = -3; delta <= 3; delta++)
 					{
-						int d=(decj+j)&7;
-
-						if (dist[d]<=minDist)
-						{
-							movement=MOV_GOING_DXDY;
-							dxdyfromDirection(d, &dx, &dy);
-							break;
-						}
+						scoreX += owner->map->getExplored(posX - 4, posY + delta, owner->teamNumber);
+						scoreX -= owner->map->getExplored(posX + 4, posY + delta, owner->teamNumber);
+						scoreY += owner->map->getExplored(posX + delta, posY - 4, owner->teamNumber);
+						scoreY -= owner->map->getExplored(posX + delta, posY + 4, owner->teamNumber);
+					}
+					int cdx, cdy;
+					simplifyDirection(scoreX, scoreY, &cdx, &cdy);
+					//printf("score = (%2d, %2d), cd = (%d, %d)\n", scoreX, scoreY, cdx, cdy);
+					if (cdx == 0 && cdy == 0)
+						movement = MOV_RANDOM_FLY;
+					else
+					{
+						dx = cdx;
+						dy = cdy;
+						directionFromDxDy();
+						movement = MOV_GOING_DXDY;
 					}
 				}
 			}
-
 			if (movement!=MOV_GOING_DXDY || owner->map->getAirUnit(posX+dx, posY+dy)!=NOGUID)
 				movement=MOV_RANDOM_FLY;
 		}
@@ -1820,6 +1891,7 @@ void Unit::handleMovement(void)
 									}
 									targetX=posX+x;
 									targetY=posY+y;
+									validTarget=true;
 									quality=newQuality;
 								}
 							}
@@ -1854,6 +1926,7 @@ void Unit::handleMovement(void)
 										}
 										targetX=posX+x;
 										targetY=posY+y;
+										validTarget=true;
 										quality=newQuality;
 									}
 								}
@@ -1879,17 +1952,64 @@ void Unit::handleMovement(void)
 			// if we haven't find anything satisfactory, follow guard area gradients
 			if (movement == MOV_RANDOM_GROUND)
 			{
-				if (owner->map->pathfindGuardArea(owner->teamNumber, (performance[SWIM]>0), posX, posY, &dx, &dy))
+				if (!attachedBuilding && owner->map->pathfindGuardArea(owner->teamNumber, (performance[SWIM]>0), posX, posY, &dx, &dy))
 				{
 					directionFromDxDy();
 					movement = MOV_GOING_DXDY;
 					// get the target position of guard area for display
 					owner->map->getGlobalGradientDestination(owner->map->guardAreasGradient[owner->teamNumber][performance[SWIM]>0], posX, posY, &targetX, &targetY);
+					validTarget=true;
+				}
+				else if (attachedBuilding || (owner->map->getGuardAreasGradient(posX, posY, performance[SWIM]>0, owner->teamNumber) == 255))
+				{
+					// are we into the guard area or war flag and we have to go to the least known area.
+					int bestExplored = 3*255;
+					int bestDirection = -1;
+					for (int di = 0; di < 8; di++)
+					{
+						int d = (direction + di) & 7;
+						int cdx, cdy;
+						dxdyfromDirection(d, &cdx, &cdy);
+						if (!owner->map->isFreeForGroundUnit(posX + cdx, posY + cdy, performance[SWIM]>0, owner->me))
+							continue;
+						if (attachedBuilding)
+						{
+							if (owner->map->warpDistSquare(posX + cdx, posY + cdy, attachedBuilding->posX, attachedBuilding->posY)
+								> ((int)attachedBuilding->unitStayRange * (int)attachedBuilding->unitStayRange))
+								continue;
+						}
+						else
+						{
+							if (owner->map->getGuardAreasGradient(posX + cdx, posY + cdy, performance[SWIM]>0, owner->teamNumber) != 255)
+								continue;
+						}
+						Uint8 explored = owner->map->getExplored(posX + 2*cdx, posY + 2*cdy, owner->teamNumber);
+						explored += owner->map->getExplored(posX + 2*cdx - cdy, posY + 2*cdy + cdx, owner->teamNumber);
+						explored += owner->map->getExplored(posX + 2*cdx + cdy, posY + 2*cdy - cdx, owner->teamNumber);
+						if (bestExplored > explored)
+						{
+							bestExplored = explored;
+							bestDirection = d;
+						}
+					}
+					if (bestDirection >= 0)
+					{
+						direction = bestDirection;
+						dxdyfromDirection();
+						movement = MOV_GOING_DXDY;
+						validTarget = false;
+					}
+					else
+					{
+						movement = MOV_RANDOM_GROUND;
+						validTarget = false;
+					}
 				}
 				else
 				{
 					// this case happens when no movement could be found because of busy places or because we are in a guard area or because there is no guard area
 					movement = MOV_RANDOM_GROUND;
+					validTarget = false;
 				}
 			}
 		}
@@ -1913,7 +2033,7 @@ void Unit::handleMovement(void)
 				{
 					int x=posX+tdx;
 					int y=posY+tdy;
-					if (map->warpDistSquare(x, y, bx, by)<=usr2 && map->isRessource(x, y, attachedBuilding->clearingRessources))
+					if (map->warpDistSquare(x, y, bx, by)<=usr2 && map->isRessourceTakeable(x, y, attachedBuilding->clearingRessources))
 					{
 						dx=tdx;
 						dy=tdy;
@@ -2584,108 +2704,108 @@ void Unit::integrity()
 	}
 }
 
-Uint32 Unit::checkSum(std::list<Uint32> *checkSumsList)
+Uint32 Unit::checkSum(std::vector<Uint32> *checkSumsVector)
 {
 	Uint32 cs=0;
 	
 	cs^=typeNum;
-	if (checkSumsList)
-		checkSumsList->push_back(typeNum);// [0]
+	if (checkSumsVector)
+		checkSumsVector->push_back(typeNum);// [0]
 	cs=(cs<<1)|(cs>>31);
 	
 	cs^=isDead;
-	if (checkSumsList)
-		checkSumsList->push_back(isDead);// [1]
+	if (checkSumsVector)
+		checkSumsVector->push_back(isDead);// [1]
 	cs=(cs<<1)|(cs>>31);
 	cs^=gid;
-	if (checkSumsList)
-		checkSumsList->push_back(gid);// [2]
+	if (checkSumsVector)
+		checkSumsVector->push_back(gid);// [2]
 	cs=(cs<<1)|(cs>>31);
 
 	cs^=posX;
-	if (checkSumsList)
-		checkSumsList->push_back(posX);// [3]
+	if (checkSumsVector)
+		checkSumsVector->push_back(posX);// [3]
 	cs=(cs<<1)|(cs>>31);
 	cs^=posY;
-	if (checkSumsList)
-		checkSumsList->push_back(posY);// [4]
+	if (checkSumsVector)
+		checkSumsVector->push_back(posY);// [4]
 	cs=(cs<<1)|(cs>>31);
 	cs^=delta;
-	if (checkSumsList)
-		checkSumsList->push_back(delta);// [5]
+	if (checkSumsVector)
+		checkSumsVector->push_back(delta);// [5]
 	cs=(cs<<1)|(cs>>31);
 	cs^=dx;
-	if (checkSumsList)
-		checkSumsList->push_back(dx);// [6]
+	if (checkSumsVector)
+		checkSumsVector->push_back(dx);// [6]
 	cs^=dy;
-	if (checkSumsList)
-		checkSumsList->push_back(dy);// [7]
+	if (checkSumsVector)
+		checkSumsVector->push_back(dy);// [7]
 	cs^=direction;
-	if (checkSumsList)
-		checkSumsList->push_back(direction);// [8]
+	if (checkSumsVector)
+		checkSumsVector->push_back(direction);// [8]
 	cs=(cs<<1)|(cs>>31);
 	cs^=insideTimeout;
-	if (checkSumsList)
-		checkSumsList->push_back(insideTimeout);// [9]
+	if (checkSumsVector)
+		checkSumsVector->push_back(insideTimeout);// [9]
 	cs=(cs<<1)|(cs>>31);
 	cs^=speed;
-	if (checkSumsList)
-		checkSumsList->push_back(speed);// [10]
+	if (checkSumsVector)
+		checkSumsVector->push_back(speed);// [10]
 	cs=(cs<<1)|(cs>>31);
 
 	cs^=(int)needToRecheckMedical;
-	if (checkSumsList)
-		checkSumsList->push_back(needToRecheckMedical);// [11]
+	if (checkSumsVector)
+		checkSumsVector->push_back(needToRecheckMedical);// [11]
 	cs=(cs<<1)|(cs>>31);
 	cs^=medical;
-	if (checkSumsList)
-		checkSumsList->push_back(medical);// [12]
+	if (checkSumsVector)
+		checkSumsVector->push_back(medical);// [12]
 	cs^=activity;
-	if (checkSumsList)
-		checkSumsList->push_back(activity);// [13]
+	if (checkSumsVector)
+		checkSumsVector->push_back(activity);// [13]
 	cs^=displacement;
-	if (checkSumsList)
-		checkSumsList->push_back(displacement);// [14]
+	if (checkSumsVector)
+		checkSumsVector->push_back(displacement);// [14]
 	cs^=movement;
-	if (checkSumsList)
-		checkSumsList->push_back(movement);// [15]
+	if (checkSumsVector)
+		checkSumsVector->push_back(movement);// [15]
 	cs^=action;
-	if (checkSumsList)
-		checkSumsList->push_back(action);// [16]
+	if (checkSumsVector)
+		checkSumsVector->push_back(action);// [16]
 	cs=(cs<<1)|(cs>>31);
 	cs^=targetX;
-	if (checkSumsList)
-		checkSumsList->push_back(targetX);// [17]
+	if (checkSumsVector)
+		checkSumsVector->push_back(targetX);// [17]
 	cs^=targetY;
-	if (checkSumsList)
-		checkSumsList->push_back(targetY);// [18]
+	if (checkSumsVector)
+		checkSumsVector->push_back(targetY);// [18]
 	cs=(cs<<1)|(cs>>31);
 
 	cs^=hp;
-	if (checkSumsList)
-		checkSumsList->push_back(hp);// [19]
+	if (checkSumsVector)
+		checkSumsVector->push_back(hp);// [19]
 	cs^=trigHP;
-	if (checkSumsList)
-		checkSumsList->push_back(trigHP);// [20]
+	if (checkSumsVector)
+		checkSumsVector->push_back(trigHP);// [20]
 	cs=(cs<<1)|(cs>>31);
 
 	cs^=hungry;
-	if (checkSumsList)
-		checkSumsList->push_back(hungry);// [21]
+	if (checkSumsVector)
+		checkSumsVector->push_back(hungry);// [21]
 	cs^=trigHungry;
-	if (checkSumsList)
-		checkSumsList->push_back(trigHungry);// [22]
+	if (checkSumsVector)
+		checkSumsVector->push_back(trigHungry);// [22]
 	cs^=trigHungryCarying;
-	if (checkSumsList)
-		checkSumsList->push_back(trigHungryCarying);// [23]
+	if (checkSumsVector)
+		checkSumsVector->push_back(trigHungryCarying);// [23]
 	cs=(cs<<1)|(cs>>31);
 	
 	cs^=fruitMask;
-	if (checkSumsList)
-		checkSumsList->push_back(fruitMask);// [24]
+	if (checkSumsVector)
+		checkSumsVector->push_back(fruitMask);// [24]
 	cs^=fruitCount;
-	if (checkSumsList)
-		checkSumsList->push_back(fruitCount);// [25]
+	if (checkSumsVector)
+		checkSumsVector->push_back(fruitCount);// [25]
 	cs=(cs<<1)|(cs>>31);
 
 	for (int i=0; i<NB_ABILITY; i++)
@@ -2697,47 +2817,47 @@ Uint32 Unit::checkSum(std::list<Uint32> *checkSumsList)
 		cs^=(Uint32)canLearn[i];
 		cs=(cs<<1)|(cs>>31);
 	}
-	if (checkSumsList)
-		checkSumsList->push_back(cs);// [26]
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [26]
 	cs=(cs<<1)|(cs>>31);
 	
 	cs^=(attachedBuilding!=NULL ? 1:0);
-	if (checkSumsList)
-		checkSumsList->push_back((attachedBuilding!=NULL ? 1:0));// [27]
+	if (checkSumsVector)
+		checkSumsVector->push_back((attachedBuilding!=NULL ? 1:0));// [27]
 	cs=(cs<<1)|(cs>>31);
 	cs^=(targetBuilding!=NULL ? 1:0);
-	if (checkSumsList)
-		checkSumsList->push_back((targetBuilding!=NULL ? 1:0));// [28]
+	if (checkSumsVector)
+		checkSumsVector->push_back((targetBuilding!=NULL ? 1:0));// [28]
 	cs^=(ownExchangeBuilding!=NULL ? 2:0);
-	if (checkSumsList)
-		checkSumsList->push_back((ownExchangeBuilding!=NULL ? 1:0));// [29]
+	if (checkSumsVector)
+		checkSumsVector->push_back((ownExchangeBuilding!=NULL ? 1:0));// [29]
 	cs^=(foreingExchangeBuilding!=NULL ? 4:0);
-	if (checkSumsList)
-		checkSumsList->push_back((foreingExchangeBuilding!=NULL ? 1:0));// [30]
+	if (checkSumsVector)
+		checkSumsVector->push_back((foreingExchangeBuilding!=NULL ? 1:0));// [30]
 	cs=(cs<<1)|(cs>>31);
 	
 	cs^=destinationPurprose;
-	if (checkSumsList)
-		checkSumsList->push_back(destinationPurprose);// [31]
+	if (checkSumsVector)
+		checkSumsVector->push_back(destinationPurprose);// [31]
 	cs^=(Uint32)subscribed;
-	if (checkSumsList)
-		checkSumsList->push_back(subscribed);// [32]
+	if (checkSumsVector)
+		checkSumsVector->push_back(subscribed);// [32]
 	cs^=caryedRessource;
-	if (checkSumsList)
-		checkSumsList->push_back(caryedRessource);// [33]
+	if (checkSumsVector)
+		checkSumsVector->push_back(caryedRessource);// [33]
 	
-	if (checkSumsList)
-		checkSumsList->push_back(0);// [34]
-	if (checkSumsList)
-		checkSumsList->push_back(0);// [35]
-	if (checkSumsList)
-		checkSumsList->push_back(0);// [36]
-	if (checkSumsList)
-		checkSumsList->push_back(0);// [37]
-	if (checkSumsList)
-		checkSumsList->push_back(0);// [38]
-	if (checkSumsList)
-		checkSumsList->push_back(0);// [39]
+	if (checkSumsVector)
+		checkSumsVector->push_back(0);// [34]
+	if (checkSumsVector)
+		checkSumsVector->push_back(0);// [35]
+	if (checkSumsVector)
+		checkSumsVector->push_back(0);// [36]
+	if (checkSumsVector)
+		checkSumsVector->push_back(0);// [37]
+	if (checkSumsVector)
+		checkSumsVector->push_back(0);// [38]
+	if (checkSumsVector)
+		checkSumsVector->push_back(0);// [39]
 	
 	return cs;
 }
