@@ -1,4 +1,4 @@
-/*
+ /*
   Copyright (C) 2001-2004 Stephane Magnenat & Luc-Olivier de Charrière
   for any question or comment contact us at nct@ysagoon.com or nuage@ysagoon.com
 
@@ -256,7 +256,7 @@ void YOG::treatPacket(IPaddress ip, Uint8 *data, int size)
 	case YMT_GAME_INFO_FROM_HOST:
 	{
 		Uint32 uid=getUint32(data, 4);
-		if (size>(4+4+4+4+64))
+		if (size>(20+64))
 			printf("Warning, bad YMT_GAME_INFO_FROM_HOST packet received from ip=(%s)\n", Utilities::stringIP(ip));
 		else
 			for (std::list<GameInfo>::iterator game=games.begin(); game!=games.end(); ++game)
@@ -269,6 +269,7 @@ void YOG::treatPacket(IPaddress ip, Uint8 *data, int size)
 					if (data[12]!=0 || data[13]!=0 || data[14]!=0)
 						printf("Warning, bad pad YMT_GAME_INFO_FROM_HOST packet received from ip=(%s)\n", Utilities::stringIP(ip));
 					game->netProtocolVersion=(int)(Sint8)data[15];
+					game->configCheckSum=getUint32(data, 16);
 					memcpy(game->mapName, data+16, size-16);
 					game->mapName[63]=0;
 					if (isSelectedGame && selectedGame==uid)
@@ -276,8 +277,7 @@ void YOG::treatPacket(IPaddress ip, Uint8 *data, int size)
 						newSelectedGameinfoAvailable=true;
 						selectedGameinfoValid=true;
 					}
-					if (verbose)
-						printf("new game->mapName=%s\n", game->mapName);
+					fprintf(logFile, "new game->mapName=%s\n", game->mapName);
 				}
 	}
 	break;
@@ -590,6 +590,7 @@ void YOG::treatPacket(IPaddress ip, Uint8 *data, int size)
 			game.fileIsAMap=false;
 			game.mapGenerationMethode=0xFF;
 			game.netProtocolVersion=0;
+			game.configCheckSum=0;
 			memset(game.mapName, 0, 64);
 			game.natSolved=false;
 			games.push_back(game);
@@ -1427,12 +1428,15 @@ void YOG::joinGame()
 	unjoiningConfirmed=false;
 }
 
-void YOG::unjoinGame(bool strict)
+void YOG::unjoinGame(bool strict, const char *reason)
 {
 	fprintf(logFile, "unjoinGame(%d) yogGlobalState=%d\n", strict, yogGlobalState);
-	assert(yogGlobalState==YGS_CONNECTED);
+	assert(yogGlobalState == YGS_CONNECTED);
 	if (strict)
 		assert(joinedGame);
+	
+	if (reason)
+		addEventMessage(reason);
 	joinedGame=false;
 	
 	joinGameSocketReceived=false;
@@ -1738,3 +1742,14 @@ bool YOG::handleLocalMessageTreatment(const char *message)
 	else
 		return false;
 }
+
+//! Add a event message with msg text to the event queue
+void YOG::addEventMessage(const char *msg)
+{
+	Message message;
+	message.gameGuiPainted = false;
+	message.messageType = YCMT_EVENT_MESSAGE;
+	strncpy(message.text, msg, sizeof(message.text));
+	message.text[sizeof(message.text) - 1] = 0;
+	receivedMessages.push_back(message);
+}
